from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class Pane(Serialisable):  # type: ignore[misc]
    xSplit: Any
    ySplit: Any
    topLeftCell: Any
    activePane: Any
    state: Any
    def __init__(
        self,
        xSplit: Any | None = ...,
        ySplit: Any | None = ...,
        topLeftCell: Any | None = ...,
        activePane: str = ...,
        state: str = ...,
    ) -> None: ...

class Selection(Serialisable):  # type: ignore[misc]
    pane: Any
    activeCell: Any
    activeCellId: Any
    sqref: Any
    def __init__(
        self, pane: Any | None = ..., activeCell: str = ..., activeCellId: Any | None = ..., sqref: str = ...
    ) -> None: ...

class SheetView(Serialisable):
    tagname: str
    windowProtection: Any
    showFormulas: Any
    showGridLines: Any
    showRowColHeaders: Any
    showZeros: Any
    rightToLeft: Any
    tabSelected: Any
    showRuler: Any
    showOutlineSymbols: Any
    defaultGridColor: Any
    showWhiteSpace: Any
    view: Any
    topLeftCell: Any
    colorId: Any
    zoomScale: Any
    zoomScaleNormal: Any
    zoomScaleSheetLayoutView: Any
    zoomScalePageLayoutView: Any
    zoomToFit: Any
    workbookViewId: Any
    selection: Any
    pane: Any
    def __init__(
        self,
        windowProtection: Any | None = ...,
        showFormulas: Any | None = ...,
        showGridLines: Any | None = ...,
        showRowColHeaders: Any | None = ...,
        showZeros: Any | None = ...,
        rightToLeft: Any | None = ...,
        tabSelected: Any | None = ...,
        showRuler: Any | None = ...,
        showOutlineSymbols: Any | None = ...,
        defaultGridColor: Any | None = ...,
        showWhiteSpace: Any | None = ...,
        view: Any | None = ...,
        topLeftCell: Any | None = ...,
        colorId: Any | None = ...,
        zoomScale: Any | None = ...,
        zoomScaleNormal: Any | None = ...,
        zoomScaleSheetLayoutView: Any | None = ...,
        zoomScalePageLayoutView: Any | None = ...,
        zoomToFit: Any | None = ...,
        workbookViewId: int = ...,
        selection: Any | None = ...,
        pane: Any | None = ...,
    ) -> None: ...

class SheetViewList(Serialisable):
    tagname: str
    sheetView: Any
    extLst: Any
    __elements__: Any
    def __init__(self, sheetView: Any | None = ..., extLst: Any | None = ...) -> None: ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

from .colors import ColorChoice

class TintEffect(Serialisable):
    tagname: str
    hue: Any
    amt: Any
    def __init__(self, hue: int = ..., amt: int = ...) -> None: ...

class LuminanceEffect(Serialisable):
    tagname: str
    bright: Any
    contrast: Any
    def __init__(self, bright: int = ..., contrast: int = ...) -> None: ...

class HSLEffect(Serialisable):
    hue: Any
    sat: Any
    lum: Any
    def __init__(self, hue: Any | None = ..., sat: Any | None = ..., lum: Any | None = ...) -> None: ...

class GrayscaleEffect(Serialisable):
    tagname: str

class FillOverlayEffect(Serialisable):
    blend: Any
    def __init__(self, blend: Any | None = ...) -> None: ...

class DuotoneEffect(Serialisable): ...
class ColorReplaceEffect(Serialisable): ...
class Color(Serialisable): ...

class ColorChangeEffect(Serialisable):
    useA: Any
    clrFrom: Any
    clrTo: Any
    def __init__(self, useA: Any | None = ..., clrFrom: Any | None = ..., clrTo: Any | None = ...) -> None: ...

class BlurEffect(Serialisable):
    rad: Any
    grow: Any
    def __init__(self, rad: Any | None = ..., grow: Any | None = ...) -> None: ...

class BiLevelEffect(Serialisable):
    thresh: Any
    def __init__(self, thresh: Any | None = ...) -> None: ...

class AlphaReplaceEffect(Serialisable):
    a: Any
    def __init__(self, a: Any | None = ...) -> None: ...

class AlphaModulateFixedEffect(Serialisable):
    amt: Any
    def __init__(self, amt: Any | None = ...) -> None: ...

class EffectContainer(Serialisable):
    type: Any
    name: Any
    def __init__(self, type: Any | None = ..., name: Any | None = ...) -> None: ...

class AlphaModulateEffect(Serialisable):
    cont: Any
    def __init__(self, cont: Any | None = ...) -> None: ...

class AlphaInverseEffect(Serialisable): ...
class AlphaFloorEffect(Serialisable): ...
class AlphaCeilingEffect(Serialisable): ...

class AlphaBiLevelEffect(Serialisable):
    thresh: Any
    def __init__(self, thresh: Any | None = ...) -> None: ...

class GlowEffect(ColorChoice):
    rad: Any
    scrgbClr: Any
    srgbClr: Any
    hslClr: Any
    sysClr: Any
    schemeClr: Any
    prstClr: Any
    __elements__: Any
    def __init__(self, rad: Any | None = ..., **kw) -> None: ...

class InnerShadowEffect(ColorChoice):
    blurRad: Any
    dist: Any
    dir: Any
    scrgbClr: Any
    srgbClr: Any
    hslClr: Any
    sysClr: Any
    schemeClr: Any
    prstClr: Any
    __elements__: Any
    def __init__(self, blurRad: Any | None = ..., dist: Any | None = ..., dir: Any | None = ..., **kw) -> None: ...

class OuterShadow(ColorChoice):
    tagname: str
    blurRad: Any
    dist: Any
    dir: Any
    sx: Any
    sy: Any
    kx: Any
    ky: Any
    algn: Any
    rotWithShape: Any
    scrgbClr: Any
    srgbClr: Any
    hslClr: Any
    sysClr: Any
    schemeClr: Any
    prstClr: Any
    __elements__: Any
    def __init__(
        self,
        blurRad: Any | None = ...,
        dist: Any | None = ...,
        dir: Any | None = ...,
        sx: Any | None = ...,
        sy: Any | None = ...,
        kx: Any | None = ...,
        ky: Any | None = ...,
        algn: Any | None = ...,
        rotWithShape: Any | None = ...,
        **kw,
    ) -> None: ...

class PresetShadowEffect(ColorChoice):
    prst: Any
    dist: Any
    dir: Any
    scrgbClr: Any
    srgbClr: Any
    hslClr: Any
    sysClr: Any
    schemeClr: Any
    prstClr: Any
    __elements__: Any
    def __init__(self, prst: Any | None = ..., dist: Any | None = ..., dir: Any | None = ..., **kw) -> None: ...

class ReflectionEffect(Serialisable):
    blurRad: Any
    stA: Any
    stPos: Any
    endA: Any
    endPos: Any
    dist: Any
    dir: Any
    fadeDir: Any
    sx: Any
    sy: Any
    kx: Any
    ky: Any
    algn: Any
    rotWithShape: Any
    def __init__(
        self,
        blurRad: Any | None = ...,
        stA: Any | None = ...,
        stPos: Any | None = ...,
        endA: Any | None = ...,
        endPos: Any | None = ...,
        dist: Any | None = ...,
        dir: Any | None = ...,
        fadeDir: Any | None = ...,
        sx: Any | None = ...,
        sy: Any | None = ...,
        kx: Any | None = ...,
        ky: Any | None = ...,
        algn: Any | None = ...,
        rotWithShape: Any | None = ...,
    ) -> None: ...

class SoftEdgesEffect(Serialisable):
    rad: Any
    def __init__(self, rad: Any | None = ...) -> None: ...

class EffectList(Serialisable):
    blur: Any
    fillOverlay: Any
    glow: Any
    innerShdw: Any
    outerShdw: Any
    prstShdw: Any
    reflection: Any
    softEdge: Any
    __elements__: Any
    def __init__(
        self,
        blur: Any | None = ...,
        fillOverlay: Any | None = ...,
        glow: Any | None = ...,
        innerShdw: Any | None = ...,
        outerShdw: Any | None = ...,
        prstShdw: Any | None = ...,
        reflection: Any | None = ...,
        softEdge: Any | None = ...,
    ) -> None: ...



import configparser
from typing import Any

from pkg_resources import Environment

def parse_bdist_wininst(name): ...
def distros_for_url(url, metadata: Any | None = ...) -> None: ...
def interpret_distro_name(
    location, basename, metadata, py_version: Any | None = ..., precedence=..., platform: Any | None = ...
) -> None: ...

class ContentChecker:
    def feed(self, block) -> None: ...
    def is_valid(self): ...
    def report(self, reporter, template) -> None: ...

class HashChecker(ContentChecker):
    pattern: Any
    hash_name: Any
    hash: Any
    expected: Any
    def __init__(self, hash_name, expected) -> None: ...
    @classmethod
    def from_url(cls, url): ...
    def feed(self, block) -> None: ...
    def is_valid(self): ...
    def report(self, reporter, template): ...

class PackageIndex(Environment):
    index_url: Any
    scanned_urls: Any
    fetched_urls: Any
    package_pages: Any
    allows: Any
    to_scan: Any
    opener: Any
    def __init__(
        self, index_url: str = ..., hosts=..., ca_bundle: Any | None = ..., verify_ssl: bool = ..., *args, **kw
    ) -> None: ...
    def process_url(self, url, retrieve: bool = ...) -> None: ...
    def process_filename(self, fn, nested: bool = ...) -> None: ...
    def url_ok(self, url, fatal: bool = ...): ...
    def scan_egg_links(self, search_path) -> None: ...
    def scan_egg_link(self, path, entry) -> None: ...
    def process_index(self, url, page): ...
    def need_version_info(self, url) -> None: ...
    def scan_all(self, msg: Any | None = ..., *args) -> None: ...
    def find_packages(self, requirement) -> None: ...
    def obtain(self, requirement, installer: Any | None = ...): ...
    def check_hash(self, checker, filename, tfp) -> None: ...
    def add_find_links(self, urls) -> None: ...
    def prescan(self) -> None: ...
    def not_found_in_index(self, requirement) -> None: ...
    def download(self, spec, tmpdir): ...
    def fetch_distribution(
        self,
        requirement,
        tmpdir,
        force_scan: bool = ...,
        source: bool = ...,
        develop_ok: bool = ...,
        local_index: Any | None = ...,
    ): ...
    def fetch(self, requirement, tmpdir, force_scan: bool = ..., source: bool = ...): ...
    def gen_setup(self, filename, fragment, tmpdir): ...
    dl_blocksize: int
    def reporthook(self, url, filename, blocknum, blksize, size) -> None: ...
    def open_url(self, url, warning: Any | None = ...): ...
    def scan_url(self, url) -> None: ...
    def debug(self, msg, *args) -> None: ...
    def info(self, msg, *args) -> None: ...
    def warn(self, msg, *args) -> None: ...

class Credential:
    username: Any
    password: Any
    def __init__(self, username, password) -> None: ...
    def __iter__(self): ...

class PyPIConfig(configparser.RawConfigParser):
    def __init__(self) -> None: ...
    @property
    def creds_by_repository(self): ...
    def find_credential(self, url): ...



from _typeshed import Self
from lib2to3.pgen2.grammar import Grammar
from typing import Any, Callable, Iterator

_NL = Node | Leaf
_Context = tuple[str, int, int]
_Results = dict[str, _NL]
_RawNode = tuple[int, str, _Context, list[_NL] | None]
_Convert = Callable[[Grammar, _RawNode], Any]

HUGE: int

def type_repr(type_num: int) -> str: ...

class Base:
    type: int
    parent: Node | None
    prefix: str
    children: list[_NL]
    was_changed: bool
    was_checked: bool
    def __eq__(self, other: object) -> bool: ...
    def _eq(self: Self, other: Self) -> bool: ...
    def clone(self: Self) -> Self: ...
    def post_order(self) -> Iterator[_NL]: ...
    def pre_order(self) -> Iterator[_NL]: ...
    def replace(self, new: _NL | list[_NL]) -> None: ...
    def get_lineno(self) -> int: ...
    def changed(self) -> None: ...
    def remove(self) -> int | None: ...
    @property
    def next_sibling(self) -> _NL | None: ...
    @property
    def prev_sibling(self) -> _NL | None: ...
    def leaves(self) -> Iterator[Leaf]: ...
    def depth(self) -> int: ...
    def get_suffix(self) -> str: ...

class Node(Base):
    fixers_applied: list[Any]
    def __init__(
        self,
        type: int,
        children: list[_NL],
        context: Any | None = ...,
        prefix: str | None = ...,
        fixers_applied: list[Any] | None = ...,
    ) -> None: ...
    def set_child(self, i: int, child: _NL) -> None: ...
    def insert_child(self, i: int, child: _NL) -> None: ...
    def append_child(self, child: _NL) -> None: ...
    def __unicode__(self) -> str: ...

class Leaf(Base):
    lineno: int
    column: int
    value: str
    fixers_applied: list[Any]
    def __init__(
        self, type: int, value: str, context: _Context | None = ..., prefix: str | None = ..., fixers_applied: list[Any] = ...
    ) -> None: ...
    def __unicode__(self) -> str: ...

def convert(gr: Grammar, raw_node: _RawNode) -> _NL: ...

class BasePattern:
    type: int
    content: str | None
    name: str | None
    def optimize(self) -> BasePattern: ...  # sic, subclasses are free to optimize themselves into different patterns
    def match(self, node: _NL, results: _Results | None = ...) -> bool: ...
    def match_seq(self, nodes: list[_NL], results: _Results | None = ...) -> bool: ...
    def generate_matches(self, nodes: list[_NL]) -> Iterator[tuple[int, _Results]]: ...

class LeafPattern(BasePattern):
    def __init__(self, type: int | None = ..., content: str | None = ..., name: str | None = ...) -> None: ...

class NodePattern(BasePattern):
    wildcards: bool
    def __init__(self, type: int | None = ..., content: str | None = ..., name: str | None = ...) -> None: ...

class WildcardPattern(BasePattern):
    min: int
    max: int
    def __init__(self, content: str | None = ..., min: int = ..., max: int = ..., name: str | None = ...) -> None: ...

class NegatedPattern(BasePattern):
    def __init__(self, content: str | None = ...) -> None: ...

def generate_matches(patterns: list[BasePattern], nodes: list[_NL]) -> Iterator[tuple[int, _Results]]: ...



from typing import Any

from openpyxl.descriptors import DateTime
from openpyxl.descriptors.nested import NestedText
from openpyxl.descriptors.serialisable import Serialisable

class NestedDateTime(DateTime, NestedText):
    expected_type: Any
    def to_tree(self, tagname: Any | None = ..., value: Any | None = ..., namespace: Any | None = ...): ...

class QualifiedDateTime(NestedDateTime):
    def to_tree(self, tagname: Any | None = ..., value: Any | None = ..., namespace: Any | None = ...): ...

class DocumentProperties(Serialisable):
    tagname: str
    namespace: Any
    category: Any
    contentStatus: Any
    keywords: Any
    lastModifiedBy: Any
    lastPrinted: Any
    revision: Any
    version: Any
    last_modified_by: Any
    subject: Any
    title: Any
    creator: Any
    description: Any
    identifier: Any
    language: Any
    created: Any
    modified: Any
    __elements__: Any
    def __init__(
        self,
        category: Any | None = ...,
        contentStatus: Any | None = ...,
        keywords: Any | None = ...,
        lastModifiedBy: Any | None = ...,
        lastPrinted: Any | None = ...,
        revision: Any | None = ...,
        version: Any | None = ...,
        created=...,
        creator: str = ...,
        description: Any | None = ...,
        identifier: Any | None = ...,
        language: Any | None = ...,
        modified=...,
        subject: Any | None = ...,
        title: Any | None = ...,
    ) -> None: ...



from _typeshed import Self
from typing import IO, Any, Text

def split(s: str | None, comments: bool = ..., posix: bool = ...) -> list[str]: ...

class shlex:
    def __init__(self, instream: IO[Any] | Text = ..., infile: IO[Any] = ..., posix: bool = ...) -> None: ...
    def __iter__(self: Self) -> Self: ...
    def next(self) -> str: ...
    def get_token(self) -> str | None: ...
    def push_token(self, _str: str) -> None: ...
    def read_token(self) -> str: ...
    def sourcehook(self, filename: str) -> None: ...
    def push_source(self, stream: IO[Any], filename: str = ...) -> None: ...
    def pop_source(self) -> IO[Any]: ...
    def error_leader(self, file: str = ..., line: int = ...) -> str: ...
    commenters: str
    wordchars: str
    whitespace: str
    escape: str
    quotes: str
    escapedquotes: str
    whitespace_split: bool
    infile: IO[Any]
    source: str | None
    debug: int
    lineno: int
    token: Any
    eof: str | None



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class WebPublishItem(Serialisable):
    tagname: str
    id: Any
    divId: Any
    sourceType: Any
    sourceRef: Any
    sourceObject: Any
    destinationFile: Any
    title: Any
    autoRepublish: Any
    def __init__(
        self,
        id: Any | None = ...,
        divId: Any | None = ...,
        sourceType: Any | None = ...,
        sourceRef: Any | None = ...,
        sourceObject: Any | None = ...,
        destinationFile: Any | None = ...,
        title: Any | None = ...,
        autoRepublish: Any | None = ...,
    ) -> None: ...

class WebPublishItems(Serialisable):
    tagname: str
    count: Any
    webPublishItem: Any
    __elements__: Any
    def __init__(self, count: Any | None = ..., webPublishItem: Any | None = ...) -> None: ...



from typing import Any, Callable, NamedTuple

from google.cloud.ndb import Key, exceptions as exceptions

class _LocalState:
    def __init__(self) -> None: ...
    @property
    def context(self): ...
    @context.setter
    def context(self, value) -> None: ...
    @property
    def toplevel_context(self): ...
    @toplevel_context.setter
    def toplevel_context(self, value) -> None: ...

def get_context(raise_context_error: bool = ...): ...
def get_toplevel_context(raise_context_error: bool = ...): ...

class _ContextTuple(NamedTuple):
    id: Any
    client: Any
    namespace: Any
    eventloop: Any
    batches: Any
    commit_batches: Any
    transaction: Any
    cache: Any
    global_cache: Any
    on_commit_callbacks: Any
    transaction_complete_callbacks: Any
    legacy_data: Any

class _Context(_ContextTuple):
    def __new__(
        cls,
        client,
        id: Any | None = ...,
        namespace=...,
        eventloop: Any | None = ...,
        batches: Any | None = ...,
        commit_batches: Any | None = ...,
        transaction: Any | None = ...,
        cache: Any | None = ...,
        cache_policy: Any | None = ...,
        global_cache: Any | None = ...,
        global_cache_policy: Callable[[Key], bool] | None = ...,
        global_cache_timeout_policy: Any | None = ...,
        datastore_policy: Any | None = ...,
        on_commit_callbacks: Any | None = ...,
        transaction_complete_callbacks: Any | None = ...,
        legacy_data: bool = ...,
        retry: Any | None = ...,
        rpc_time: Any | None = ...,
        wait_time: Any | None = ...,
    ): ...
    def new(self, **kwargs): ...
    rpc_time: int
    wait_time: int
    def use(self) -> None: ...

class Context(_Context):
    def clear_cache(self) -> None: ...
    def flush(self) -> None: ...
    def get_namespace(self): ...
    def get_cache_policy(self): ...
    def get_datastore_policy(self) -> None: ...
    def get_global_cache_policy(self): ...
    get_memcache_policy: Any
    def get_global_cache_timeout_policy(self): ...
    get_memcache_timeout_policy: Any
    cache_policy: Any
    def set_cache_policy(self, policy): ...
    datastore_policy: Any
    def set_datastore_policy(self, policy): ...
    global_cache_policy: Any
    def set_global_cache_policy(self, policy): ...
    set_memcache_policy: Any
    global_cache_timeout_policy: Any
    def set_global_cache_timeout_policy(self, policy): ...
    set_memcache_timeout_policy: Any
    def get_retry_state(self): ...
    def set_retry_state(self, state) -> None: ...
    def clear_retry_state(self) -> None: ...
    def call_on_commit(self, callback) -> None: ...
    def in_transaction(self): ...
    def in_retry(self): ...
    def memcache_add(self, *args, **kwargs) -> None: ...
    def memcache_cas(self, *args, **kwargs) -> None: ...
    def memcache_decr(self, *args, **kwargs) -> None: ...
    def memcache_delete(self, *args, **kwargs) -> None: ...
    def memcache_get(self, *args, **kwargs) -> None: ...
    def memcache_gets(self, *args, **kwargs) -> None: ...
    def memcache_incr(self, *args, **kwargs) -> None: ...
    def memcache_replace(self, *args, **kwargs) -> None: ...
    def memcache_set(self, *args, **kwargs) -> None: ...
    def urlfetch(self, *args, **kwargs) -> None: ...

class ContextOptions:
    def __init__(self, *args, **kwargs) -> None: ...

class TransactionOptions:
    NESTED: int
    MANDATORY: int
    ALLOWED: int
    INDEPENDENT: int

class AutoBatcher:
    def __init__(self, *args, **kwargs) -> None: ...



import collections  # Needed by aliases like DefaultDict, see mypy issue 2986
import sys
from _typeshed import Self as TypeshedSelf, SupportsKeysAndGetItem
from abc import ABCMeta, abstractmethod
from types import BuiltinFunctionType, CodeType, FrameType, FunctionType, MethodType, ModuleType, TracebackType
from typing_extensions import Literal as _Literal, ParamSpec as _ParamSpec, final as _final

if sys.version_info >= (3, 7):
    from types import MethodDescriptorType, MethodWrapperType, WrapperDescriptorType

if sys.version_info >= (3, 9):
    from types import GenericAlias

if sys.version_info >= (3, 11):
    __all__ = [
        "Annotated",
        "Any",
        "Callable",
        "ClassVar",
        "Concatenate",
        "Final",
        "ForwardRef",
        "Generic",
        "Literal",
        "Optional",
        "ParamSpec",
        "Protocol",
        "Tuple",
        "Type",
        "TypeVar",
        "TypeVarTuple",
        "Union",
        "Unpack",
        "AbstractSet",
        "ByteString",
        "Container",
        "ContextManager",
        "Hashable",
        "ItemsView",
        "Iterable",
        "Iterator",
        "KeysView",
        "Mapping",
        "MappingView",
        "MutableMapping",
        "MutableSequence",
        "MutableSet",
        "Sequence",
        "Sized",
        "ValuesView",
        "Awaitable",
        "AsyncIterator",
        "AsyncIterable",
        "Coroutine",
        "Collection",
        "AsyncGenerator",
        "AsyncContextManager",
        "Reversible",
        "SupportsAbs",
        "SupportsBytes",
        "SupportsComplex",
        "SupportsFloat",
        "SupportsIndex",
        "SupportsInt",
        "SupportsRound",
        "ChainMap",
        "Counter",
        "Deque",
        "Dict",
        "DefaultDict",
        "List",
        "OrderedDict",
        "Set",
        "FrozenSet",
        "NamedTuple",
        "TypedDict",
        "Generator",
        "BinaryIO",
        "IO",
        "Match",
        "Pattern",
        "TextIO",
        "AnyStr",
        "assert_never",
        "assert_type",
        "cast",
        "final",
        "get_args",
        "get_origin",
        "get_type_hints",
        "is_typeddict",
        "Never",
        "NewType",
        "no_type_check",
        "no_type_check_decorator",
        "NoReturn",
        "overload",
        "ParamSpecArgs",
        "ParamSpecKwargs",
        "reveal_type",
        "runtime_checkable",
        "Self",
        "Text",
        "TYPE_CHECKING",
        "TypeAlias",
        "TypeGuard",
    ]
elif sys.version_info >= (3, 10):
    __all__ = [
        "Annotated",
        "Any",
        "Callable",
        "ClassVar",
        "Concatenate",
        "Final",
        "ForwardRef",
        "Generic",
        "Literal",
        "Optional",
        "ParamSpec",
        "Protocol",
        "Tuple",
        "Type",
        "TypeVar",
        "Union",
        "AbstractSet",
        "ByteString",
        "Container",
        "ContextManager",
        "Hashable",
        "ItemsView",
        "Iterable",
        "Iterator",
        "KeysView",
        "Mapping",
        "MappingView",
        "MutableMapping",
        "MutableSequence",
        "MutableSet",
        "Sequence",
        "Sized",
        "ValuesView",
        "Awaitable",
        "AsyncIterator",
        "AsyncIterable",
        "Coroutine",
        "Collection",
        "AsyncGenerator",
        "AsyncContextManager",
        "Reversible",
        "SupportsAbs",
        "SupportsBytes",
        "SupportsComplex",
        "SupportsFloat",
        "SupportsIndex",
        "SupportsInt",
        "SupportsRound",
        "ChainMap",
        "Counter",
        "Deque",
        "Dict",
        "DefaultDict",
        "List",
        "OrderedDict",
        "Set",
        "FrozenSet",
        "NamedTuple",
        "TypedDict",
        "Generator",
        "BinaryIO",
        "IO",
        "Match",
        "Pattern",
        "TextIO",
        "AnyStr",
        "cast",
        "final",
        "get_args",
        "get_origin",
        "get_type_hints",
        "is_typeddict",
        "NewType",
        "no_type_check",
        "no_type_check_decorator",
        "NoReturn",
        "overload",
        "ParamSpecArgs",
        "ParamSpecKwargs",
        "runtime_checkable",
        "Text",
        "TYPE_CHECKING",
        "TypeAlias",
        "TypeGuard",
    ]
elif sys.version_info >= (3, 9):
    __all__ = [
        "Annotated",
        "Any",
        "Callable",
        "ClassVar",
        "Final",
        "ForwardRef",
        "Generic",
        "Literal",
        "Optional",
        "Protocol",
        "Tuple",
        "Type",
        "TypeVar",
        "Union",
        "AbstractSet",
        "ByteString",
        "Container",
        "ContextManager",
        "Hashable",
        "ItemsView",
        "Iterable",
        "Iterator",
        "KeysView",
        "Mapping",
        "MappingView",
        "MutableMapping",
        "MutableSequence",
        "MutableSet",
        "Sequence",
        "Sized",
        "ValuesView",
        "Awaitable",
        "AsyncIterator",
        "AsyncIterable",
        "Coroutine",
        "Collection",
        "AsyncGenerator",
        "AsyncContextManager",
        "Reversible",
        "SupportsAbs",
        "SupportsBytes",
        "SupportsComplex",
        "SupportsFloat",
        "SupportsIndex",
        "SupportsInt",
        "SupportsRound",
        "ChainMap",
        "Counter",
        "Deque",
        "Dict",
        "DefaultDict",
        "List",
        "OrderedDict",
        "Set",
        "FrozenSet",
        "NamedTuple",
        "TypedDict",
        "Generator",
        "BinaryIO",
        "IO",
        "Match",
        "Pattern",
        "TextIO",
        "AnyStr",
        "cast",
        "final",
        "get_args",
        "get_origin",
        "get_type_hints",
        "NewType",
        "no_type_check",
        "no_type_check_decorator",
        "NoReturn",
        "overload",
        "runtime_checkable",
        "Text",
        "TYPE_CHECKING",
    ]
elif sys.version_info >= (3, 8):
    __all__ = [
        "Any",
        "Callable",
        "ClassVar",
        "Final",
        "ForwardRef",
        "Generic",
        "Literal",
        "Optional",
        "Protocol",
        "Tuple",
        "Type",
        "TypeVar",
        "Union",
        "AbstractSet",
        "ByteString",
        "Container",
        "ContextManager",
        "Hashable",
        "ItemsView",
        "Iterable",
        "Iterator",
        "KeysView",
        "Mapping",
        "MappingView",
        "MutableMapping",
        "MutableSequence",
        "MutableSet",
        "Sequence",
        "Sized",
        "ValuesView",
        "Awaitable",
        "AsyncIterator",
        "AsyncIterable",
        "Coroutine",
        "Collection",
        "AsyncGenerator",
        "AsyncContextManager",
        "Reversible",
        "SupportsAbs",
        "SupportsBytes",
        "SupportsComplex",
        "SupportsFloat",
        "SupportsIndex",
        "SupportsInt",
        "SupportsRound",
        "ChainMap",
        "Counter",
        "Deque",
        "Dict",
        "DefaultDict",
        "List",
        "OrderedDict",
        "Set",
        "FrozenSet",
        "NamedTuple",
        "TypedDict",
        "Generator",
        "AnyStr",
        "cast",
        "final",
        "get_args",
        "get_origin",
        "get_type_hints",
        "NewType",
        "no_type_check",
        "no_type_check_decorator",
        "NoReturn",
        "overload",
        "runtime_checkable",
        "Text",
        "TYPE_CHECKING",
    ]
elif sys.version_info >= (3, 7):
    __all__ = [
        "Any",
        "Callable",
        "ClassVar",
        "ForwardRef",
        "Generic",
        "Optional",
        "Tuple",
        "Type",
        "TypeVar",
        "Union",
        "AbstractSet",
        "ByteString",
        "Container",
        "ContextManager",
        "Hashable",
        "ItemsView",
        "Iterable",
        "Iterator",
        "KeysView",
        "Mapping",
        "MappingView",
        "MutableMapping",
        "MutableSequence",
        "MutableSet",
        "Sequence",
        "Sized",
        "ValuesView",
        "Awaitable",
        "AsyncIterator",
        "AsyncIterable",
        "Coroutine",
        "Collection",
        "AsyncGenerator",
        "AsyncContextManager",
        "Reversible",
        "SupportsAbs",
        "SupportsBytes",
        "SupportsComplex",
        "SupportsFloat",
        "SupportsInt",
        "SupportsRound",
        "ChainMap",
        "Counter",
        "Deque",
        "Dict",
        "DefaultDict",
        "List",
        "OrderedDict",
        "Set",
        "FrozenSet",
        "NamedTuple",
        "Generator",
        "AnyStr",
        "cast",
        "get_type_hints",
        "NewType",
        "no_type_check",
        "no_type_check_decorator",
        "NoReturn",
        "overload",
        "Text",
        "TYPE_CHECKING",
    ]
else:
    __all__ = [
        "Any",
        "Callable",
        "ClassVar",
        "Generic",
        "Optional",
        "Tuple",
        "Type",
        "TypeVar",
        "Union",
        "AbstractSet",
        "GenericMeta",
        "ByteString",
        "Container",
        "ContextManager",
        "Hashable",
        "ItemsView",
        "Iterable",
        "Iterator",
        "KeysView",
        "Mapping",
        "MappingView",
        "MutableMapping",
        "MutableSequence",
        "MutableSet",
        "Sequence",
        "Sized",
        "ValuesView",
        "Reversible",
        "SupportsAbs",
        "SupportsBytes",
        "SupportsComplex",
        "SupportsFloat",
        "SupportsInt",
        "SupportsRound",
        "Counter",
        "Deque",
        "Dict",
        "DefaultDict",
        "List",
        "Set",
        "FrozenSet",
        "NamedTuple",
        "Generator",
        "AnyStr",
        "cast",
        "get_type_hints",
        "NewType",
        "no_type_check",
        "no_type_check_decorator",
        "overload",
        "Text",
        "TYPE_CHECKING",
    ]

Any = object()

@_final
class TypeVar:
    __name__: str
    __bound__: Any | None
    __constraints__: tuple[Any, ...]
    __covariant__: bool
    __contravariant__: bool
    def __init__(
        self, name: str, *constraints: Any, bound: Any | None = ..., covariant: bool = ..., contravariant: bool = ...
    ) -> None: ...
    if sys.version_info >= (3, 10):
        def __or__(self, right: Any) -> _SpecialForm: ...
        def __ror__(self, left: Any) -> _SpecialForm: ...

# Used for an undocumented mypy feature. Does not exist at runtime.
_promote = object()

# N.B. Keep this definition in sync with typing_extensions._SpecialForm
@_final
class _SpecialForm:
    def __getitem__(self, parameters: Any) -> object: ...
    if sys.version_info >= (3, 10):
        def __or__(self, other: Any) -> _SpecialForm: ...
        def __ror__(self, other: Any) -> _SpecialForm: ...

_F = TypeVar("_F", bound=Callable[..., Any])
_P = _ParamSpec("_P")
_T = TypeVar("_T")

def overload(func: _F) -> _F: ...

# Unlike the vast majority module-level objects in stub files,
# these `_SpecialForm` objects in typing need the default value `= ...`,
# due to the fact that they are used elswhere in the same file.
# Otherwise, flake8 erroneously flags them as undefined.
# `_SpecialForm` objects in typing.py that are not used elswhere in the same file
# do not need the default value assignment.
Union: _SpecialForm = ...
Generic: _SpecialForm = ...
# Protocol is only present in 3.8 and later, but mypy needs it unconditionally
Protocol: _SpecialForm = ...
Callable: _SpecialForm = ...
Type: _SpecialForm = ...
NoReturn: _SpecialForm = ...

Optional: _SpecialForm
Tuple: _SpecialForm
ClassVar: _SpecialForm
if sys.version_info >= (3, 8):
    Final: _SpecialForm
    def final(f: _T) -> _T: ...
    Literal: _SpecialForm
    # TypedDict is a (non-subscriptable) special form.
    TypedDict: object

if sys.version_info >= (3, 11):
    Self: _SpecialForm
    Never: _SpecialForm = ...
    Unpack: _SpecialForm

    class TypeVarTuple:
        __name__: str
        def __init__(self, name: str) -> None: ...
        def __iter__(self) -> Any: ...

if sys.version_info < (3, 7):
    class GenericMeta(type): ...

if sys.version_info >= (3, 10):
    class ParamSpecArgs:
        __origin__: ParamSpec
        def __init__(self, origin: ParamSpec) -> None: ...

    class ParamSpecKwargs:
        __origin__: ParamSpec
        def __init__(self, origin: ParamSpec) -> None: ...

    class ParamSpec:
        __name__: str
        __bound__: Any | None
        __covariant__: bool
        __contravariant__: bool
        def __init__(self, name: str, *, bound: Any | None = ..., contravariant: bool = ..., covariant: bool = ...) -> None: ...
        @property
        def args(self) -> ParamSpecArgs: ...
        @property
        def kwargs(self) -> ParamSpecKwargs: ...
        def __or__(self, right: Any) -> _SpecialForm: ...
        def __ror__(self, left: Any) -> _SpecialForm: ...
    Concatenate: _SpecialForm
    TypeAlias: _SpecialForm
    TypeGuard: _SpecialForm

    class NewType:
        def __init__(self, name: str, tp: Any) -> None: ...
        def __call__(self, x: _T) -> _T: ...
        def __or__(self, other: Any) -> _SpecialForm: ...
        def __ror__(self, other: Any) -> _SpecialForm: ...
        __supertype__: type

else:
    def NewType(name: str, tp: Any) -> Any: ...

# These type variables are used by the container types.
_S = TypeVar("_S")
_KT = TypeVar("_KT")  # Key type.
_VT = TypeVar("_VT")  # Value type.
_T_co = TypeVar("_T_co", covariant=True)  # Any type covariant containers.
_V_co = TypeVar("_V_co", covariant=True)  # Any type covariant containers.
_KT_co = TypeVar("_KT_co", covariant=True)  # Key type covariant containers.
_VT_co = TypeVar("_VT_co", covariant=True)  # Value type covariant containers.
_T_contra = TypeVar("_T_contra", contravariant=True)  # Ditto contravariant.
_TC = TypeVar("_TC", bound=Type[object])

def no_type_check(arg: _F) -> _F: ...
def no_type_check_decorator(decorator: Callable[_P, _T]) -> Callable[_P, _T]: ...  # type: ignore[misc]

# Type aliases and type constructors

class _Alias:
    # Class for defining generic aliases for library types.
    def __getitem__(self, typeargs: Any) -> Any: ...

List = _Alias()
Dict = _Alias()
DefaultDict = _Alias()
Set = _Alias()
FrozenSet = _Alias()
Counter = _Alias()
Deque = _Alias()
ChainMap = _Alias()

if sys.version_info >= (3, 7):
    OrderedDict = _Alias()

if sys.version_info >= (3, 9):
    Annotated: _SpecialForm

# Predefined type variables.
AnyStr = TypeVar("AnyStr", str, bytes)  # noqa: Y001

# Technically in 3.7 this inherited from GenericMeta. But let's not reflect that, since
# type checkers tend to assume that Protocols all have the ABCMeta metaclass.
class _ProtocolMeta(ABCMeta): ...

# Abstract base classes.

def runtime_checkable(cls: _TC) -> _TC: ...
@runtime_checkable
class SupportsInt(Protocol, metaclass=ABCMeta):
    @abstractmethod
    def __int__(self) -> int: ...

@runtime_checkable
class SupportsFloat(Protocol, metaclass=ABCMeta):
    @abstractmethod
    def __float__(self) -> float: ...

@runtime_checkable
class SupportsComplex(Protocol, metaclass=ABCMeta):
    @abstractmethod
    def __complex__(self) -> complex: ...

@runtime_checkable
class SupportsBytes(Protocol, metaclass=ABCMeta):
    @abstractmethod
    def __bytes__(self) -> bytes: ...

if sys.version_info >= (3, 8):
    @runtime_checkable
    class SupportsIndex(Protocol, metaclass=ABCMeta):
        @abstractmethod
        def __index__(self) -> int: ...

@runtime_checkable
class SupportsAbs(Protocol[_T_co]):
    @abstractmethod
    def __abs__(self) -> _T_co: ...

@runtime_checkable
class SupportsRound(Protocol[_T_co]):
    @overload
    @abstractmethod
    def __round__(self) -> int: ...
    @overload
    @abstractmethod
    def __round__(self, __ndigits: int) -> _T_co: ...

@runtime_checkable
class Sized(Protocol, metaclass=ABCMeta):
    @abstractmethod
    def __len__(self) -> int: ...

@runtime_checkable
class Hashable(Protocol, metaclass=ABCMeta):
    # TODO: This is special, in that a subclass of a hashable class may not be hashable
    #   (for example, list vs. object). It's not obvious how to represent this. This class
    #   is currently mostly useless for static checking.
    @abstractmethod
    def __hash__(self) -> int: ...

@runtime_checkable
class Iterable(Protocol[_T_co]):
    @abstractmethod
    def __iter__(self) -> Iterator[_T_co]: ...

@runtime_checkable
class Iterator(Iterable[_T_co], Protocol[_T_co]):
    @abstractmethod
    def __next__(self) -> _T_co: ...
    def __iter__(self) -> Iterator[_T_co]: ...

@runtime_checkable
class Reversible(Iterable[_T_co], Protocol[_T_co]):
    @abstractmethod
    def __reversed__(self) -> Iterator[_T_co]: ...

class Generator(Iterator[_T_co], Generic[_T_co, _T_contra, _V_co]):
    def __next__(self) -> _T_co: ...
    @abstractmethod
    def send(self, __value: _T_contra) -> _T_co: ...
    @overload
    @abstractmethod
    def throw(
        self, __typ: Type[BaseException], __val: BaseException | object = ..., __tb: TracebackType | None = ...
    ) -> _T_co: ...
    @overload
    @abstractmethod
    def throw(self, __typ: BaseException, __val: None = ..., __tb: TracebackType | None = ...) -> _T_co: ...
    def close(self) -> None: ...
    def __iter__(self) -> Generator[_T_co, _T_contra, _V_co]: ...
    @property
    def gi_code(self) -> CodeType: ...
    @property
    def gi_frame(self) -> FrameType: ...
    @property
    def gi_running(self) -> bool: ...
    @property
    def gi_yieldfrom(self) -> Generator[Any, Any, Any] | None: ...

@runtime_checkable
class Awaitable(Protocol[_T_co]):
    @abstractmethod
    def __await__(self) -> Generator[Any, None, _T_co]: ...

class Coroutine(Awaitable[_V_co], Generic[_T_co, _T_contra, _V_co]):
    __name__: str
    __qualname__: str
    @property
    def cr_await(self) -> Any | None: ...
    @property
    def cr_code(self) -> CodeType: ...
    @property
    def cr_frame(self) -> FrameType: ...
    @property
    def cr_running(self) -> bool: ...
    @abstractmethod
    def send(self, __value: _T_contra) -> _T_co: ...
    @overload
    @abstractmethod
    def throw(
        self, __typ: Type[BaseException], __val: BaseException | object = ..., __tb: TracebackType | None = ...
    ) -> _T_co: ...
    @overload
    @abstractmethod
    def throw(self, __typ: BaseException, __val: None = ..., __tb: TracebackType | None = ...) -> _T_co: ...
    @abstractmethod
    def close(self) -> None: ...

# NOTE: This type does not exist in typing.py or PEP 484 but mypy needs it to exist.
# The parameters correspond to Generator, but the 4th is the original type.
class AwaitableGenerator(
    Awaitable[_V_co], Generator[_T_co, _T_contra, _V_co], Generic[_T_co, _T_contra, _V_co, _S], metaclass=ABCMeta
): ...

@runtime_checkable
class AsyncIterable(Protocol[_T_co]):
    @abstractmethod
    def __aiter__(self) -> AsyncIterator[_T_co]: ...

@runtime_checkable
class AsyncIterator(AsyncIterable[_T_co], Protocol[_T_co]):
    @abstractmethod
    def __anext__(self) -> Awaitable[_T_co]: ...
    def __aiter__(self) -> AsyncIterator[_T_co]: ...

class AsyncGenerator(AsyncIterator[_T_co], Generic[_T_co, _T_contra]):
    def __anext__(self) -> Awaitable[_T_co]: ...
    @abstractmethod
    def asend(self, __value: _T_contra) -> Awaitable[_T_co]: ...
    @overload
    @abstractmethod
    def athrow(
        self, __typ: Type[BaseException], __val: BaseException | object = ..., __tb: TracebackType | None = ...
    ) -> Awaitable[_T_co]: ...
    @overload
    @abstractmethod
    def athrow(self, __typ: BaseException, __val: None = ..., __tb: TracebackType | None = ...) -> Awaitable[_T_co]: ...
    def aclose(self) -> Awaitable[None]: ...
    @property
    def ag_await(self) -> Any: ...
    @property
    def ag_code(self) -> CodeType: ...
    @property
    def ag_frame(self) -> FrameType: ...
    @property
    def ag_running(self) -> bool: ...

@runtime_checkable
class Container(Protocol[_T_co]):
    @abstractmethod
    def __contains__(self, __x: object) -> bool: ...

@runtime_checkable
class Collection(Iterable[_T_co], Container[_T_co], Protocol[_T_co]):
    # Implement Sized (but don't have it as a base class).
    @abstractmethod
    def __len__(self) -> int: ...

class Sequence(Collection[_T_co], Reversible[_T_co], Generic[_T_co]):
    @overload
    @abstractmethod
    def __getitem__(self, index: int) -> _T_co: ...
    @overload
    @abstractmethod
    def __getitem__(self, index: slice) -> Sequence[_T_co]: ...
    # Mixin methods
    def index(self, value: Any, start: int = ..., stop: int = ...) -> int: ...
    def count(self, value: Any) -> int: ...
    def __contains__(self, value: object) -> bool: ...
    def __iter__(self) -> Iterator[_T_co]: ...
    def __reversed__(self) -> Iterator[_T_co]: ...

class MutableSequence(Sequence[_T], Generic[_T]):
    @abstractmethod
    def insert(self, index: int, value: _T) -> None: ...
    @overload
    @abstractmethod
    def __getitem__(self, index: int) -> _T: ...
    @overload
    @abstractmethod
    def __getitem__(self, index: slice) -> MutableSequence[_T]: ...
    @overload
    @abstractmethod
    def __setitem__(self, index: int, value: _T) -> None: ...
    @overload
    @abstractmethod
    def __setitem__(self, index: slice, value: Iterable[_T]) -> None: ...
    @overload
    @abstractmethod
    def __delitem__(self, index: int) -> None: ...
    @overload
    @abstractmethod
    def __delitem__(self, index: slice) -> None: ...
    # Mixin methods
    def append(self, value: _T) -> None: ...
    def clear(self) -> None: ...
    def extend(self, values: Iterable[_T]) -> None: ...
    def reverse(self) -> None: ...
    def pop(self, index: int = ...) -> _T: ...
    def remove(self, value: _T) -> None: ...
    def __iadd__(self: TypeshedSelf, values: Iterable[_T]) -> TypeshedSelf: ...

class AbstractSet(Collection[_T_co], Generic[_T_co]):
    @abstractmethod
    def __contains__(self, x: object) -> bool: ...
    def _hash(self) -> int: ...
    # Mixin methods
    def __le__(self, other: AbstractSet[Any]) -> bool: ...
    def __lt__(self, other: AbstractSet[Any]) -> bool: ...
    def __gt__(self, other: AbstractSet[Any]) -> bool: ...
    def __ge__(self, other: AbstractSet[Any]) -> bool: ...
    def __and__(self, other: AbstractSet[Any]) -> AbstractSet[_T_co]: ...
    def __or__(self, other: AbstractSet[_T]) -> AbstractSet[_T_co | _T]: ...
    def __sub__(self, other: AbstractSet[Any]) -> AbstractSet[_T_co]: ...
    def __xor__(self, other: AbstractSet[_T]) -> AbstractSet[_T_co | _T]: ...
    def isdisjoint(self, other: Iterable[Any]) -> bool: ...

class MutableSet(AbstractSet[_T], Generic[_T]):
    @abstractmethod
    def add(self, value: _T) -> None: ...
    @abstractmethod
    def discard(self, value: _T) -> None: ...
    # Mixin methods
    def clear(self) -> None: ...
    def pop(self) -> _T: ...
    def remove(self, value: _T) -> None: ...
    def __ior__(self: TypeshedSelf, it: AbstractSet[_T]) -> TypeshedSelf: ...  # type: ignore[override,misc]
    def __iand__(self: TypeshedSelf, it: AbstractSet[Any]) -> TypeshedSelf: ...
    def __ixor__(self: TypeshedSelf, it: AbstractSet[_T]) -> TypeshedSelf: ...  # type: ignore[override,misc]
    def __isub__(self: TypeshedSelf, it: AbstractSet[Any]) -> TypeshedSelf: ...

class MappingView(Sized):
    def __init__(self, mapping: Mapping[Any, Any]) -> None: ...  # undocumented
    def __len__(self) -> int: ...

class ItemsView(MappingView, AbstractSet[tuple[_KT_co, _VT_co]], Generic[_KT_co, _VT_co]):
    def __init__(self, mapping: Mapping[_KT_co, _VT_co]) -> None: ...  # undocumented
    def __and__(self, other: Iterable[Any]) -> set[tuple[_KT_co, _VT_co]]: ...
    def __rand__(self, other: Iterable[_T]) -> set[_T]: ...
    def __contains__(self, item: object) -> bool: ...
    def __iter__(self) -> Iterator[tuple[_KT_co, _VT_co]]: ...
    if sys.version_info >= (3, 8):
        def __reversed__(self) -> Iterator[tuple[_KT_co, _VT_co]]: ...

    def __or__(self, other: Iterable[_T]) -> set[tuple[_KT_co, _VT_co] | _T]: ...
    def __ror__(self, other: Iterable[_T]) -> set[tuple[_KT_co, _VT_co] | _T]: ...
    def __sub__(self, other: Iterable[Any]) -> set[tuple[_KT_co, _VT_co]]: ...
    def __rsub__(self, other: Iterable[_T]) -> set[_T]: ...
    def __xor__(self, other: Iterable[_T]) -> set[tuple[_KT_co, _VT_co] | _T]: ...
    def __rxor__(self, other: Iterable[_T]) -> set[tuple[_KT_co, _VT_co] | _T]: ...

class KeysView(MappingView, AbstractSet[_KT_co], Generic[_KT_co]):
    def __init__(self, mapping: Mapping[_KT_co, Any]) -> None: ...  # undocumented
    def __and__(self, other: Iterable[Any]) -> set[_KT_co]: ...
    def __rand__(self, other: Iterable[_T]) -> set[_T]: ...
    def __contains__(self, key: object) -> bool: ...
    def __iter__(self) -> Iterator[_KT_co]: ...
    if sys.version_info >= (3, 8):
        def __reversed__(self) -> Iterator[_KT_co]: ...

    def __or__(self, other: Iterable[_T]) -> set[_KT_co | _T]: ...
    def __ror__(self, other: Iterable[_T]) -> set[_KT_co | _T]: ...
    def __sub__(self, other: Iterable[Any]) -> set[_KT_co]: ...
    def __rsub__(self, other: Iterable[_T]) -> set[_T]: ...
    def __xor__(self, other: Iterable[_T]) -> set[_KT_co | _T]: ...
    def __rxor__(self, other: Iterable[_T]) -> set[_KT_co | _T]: ...

class ValuesView(MappingView, Iterable[_VT_co], Generic[_VT_co]):
    def __init__(self, mapping: Mapping[Any, _VT_co]) -> None: ...  # undocumented
    def __contains__(self, value: object) -> bool: ...
    def __iter__(self) -> Iterator[_VT_co]: ...
    if sys.version_info >= (3, 8):
        def __reversed__(self) -> Iterator[_VT_co]: ...

@runtime_checkable
class ContextManager(Protocol[_T_co]):
    def __enter__(self) -> _T_co: ...
    @abstractmethod
    def __exit__(
        self, __exc_type: Type[BaseException] | None, __exc_value: BaseException | None, __traceback: TracebackType | None
    ) -> bool | None: ...

@runtime_checkable
class AsyncContextManager(Protocol[_T_co]):
    async def __aenter__(self) -> _T_co: ...
    @abstractmethod
    async def __aexit__(
        self, __exc_type: Type[BaseException] | None, __exc_value: BaseException | None, __traceback: TracebackType | None
    ) -> bool | None: ...

class Mapping(Collection[_KT], Generic[_KT, _VT_co]):
    # TODO: We wish the key type could also be covariant, but that doesn't work,
    # see discussion in https://github.com/python/typing/pull/273.
    @abstractmethod
    def __getitem__(self, __k: _KT) -> _VT_co: ...
    # Mixin methods
    @overload
    def get(self, __key: _KT) -> _VT_co | None: ...
    @overload
    def get(self, __key: _KT, default: _VT_co | _T) -> _VT_co | _T: ...
    def items(self) -> ItemsView[_KT, _VT_co]: ...
    def keys(self) -> KeysView[_KT]: ...
    def values(self) -> ValuesView[_VT_co]: ...
    def __contains__(self, __o: object) -> bool: ...

class MutableMapping(Mapping[_KT, _VT], Generic[_KT, _VT]):
    @abstractmethod
    def __setitem__(self, __k: _KT, __v: _VT) -> None: ...
    @abstractmethod
    def __delitem__(self, __v: _KT) -> None: ...
    def clear(self) -> None: ...
    @overload
    def pop(self, __key: _KT) -> _VT: ...
    @overload
    def pop(self, __key: _KT, default: _VT | _T) -> _VT | _T: ...
    def popitem(self) -> tuple[_KT, _VT]: ...
    # This overload should be allowed only if the value type is compatible with None.
    # Keep OrderedDict.setdefault in line with MutableMapping.setdefault, modulo positional-only differences.
    @overload
    def setdefault(self: MutableMapping[_KT, _T | None], __key: _KT) -> _T | None: ...
    @overload
    def setdefault(self, __key: _KT, __default: _VT) -> _VT: ...
    # 'update' used to take a Union, but using overloading is better.
    # The second overloaded type here is a bit too general, because
    # Mapping[tuple[_KT, _VT], W] is a subclass of Iterable[tuple[_KT, _VT]],
    # but will always have the behavior of the first overloaded type
    # at runtime, leading to keys of a mix of types _KT and tuple[_KT, _VT].
    # We don't currently have any way of forcing all Mappings to use
    # the first overload, but by using overloading rather than a Union,
    # mypy will commit to using the first overload when the argument is
    # known to be a Mapping with unknown type parameters, which is closer
    # to the behavior we want. See mypy issue  #1430.
    #
    # Various mapping classes have __ior__ methods that should be kept roughly in line with .update():
    # -- dict.__ior__
    # -- os._Environ.__ior__
    # -- collections.UserDict.__ior__
    # -- collections.ChainMap.__ior__
    # -- weakref.WeakValueDictionary.__ior__
    # -- weakref.WeakKeyDictionary.__ior__
    @overload
    def update(self, __m: SupportsKeysAndGetItem[_KT, _VT], **kwargs: _VT) -> None: ...
    @overload
    def update(self, __m: Iterable[tuple[_KT, _VT]], **kwargs: _VT) -> None: ...
    @overload
    def update(self, **kwargs: _VT) -> None: ...

Text = str

TYPE_CHECKING: bool

# In stubs, the arguments of the IO class are marked as positional-only.
# This differs from runtime, but better reflects the fact that in reality
# classes deriving from IO use different names for the arguments.
class IO(Iterator[AnyStr], Generic[AnyStr]):
    # TODO use abstract properties
    @property
    def mode(self) -> str: ...
    @property
    def name(self) -> str: ...
    @abstractmethod
    def close(self) -> None: ...
    @property
    def closed(self) -> bool: ...
    @abstractmethod
    def fileno(self) -> int: ...
    @abstractmethod
    def flush(self) -> None: ...
    @abstractmethod
    def isatty(self) -> bool: ...
    @abstractmethod
    def read(self, __n: int = ...) -> AnyStr: ...
    @abstractmethod
    def readable(self) -> bool: ...
    @abstractmethod
    def readline(self, __limit: int = ...) -> AnyStr: ...
    @abstractmethod
    def readlines(self, __hint: int = ...) -> list[AnyStr]: ...
    @abstractmethod
    def seek(self, __offset: int, __whence: int = ...) -> int: ...
    @abstractmethod
    def seekable(self) -> bool: ...
    @abstractmethod
    def tell(self) -> int: ...
    @abstractmethod
    def truncate(self, __size: int | None = ...) -> int: ...
    @abstractmethod
    def writable(self) -> bool: ...
    @abstractmethod
    def write(self, __s: AnyStr) -> int: ...
    @abstractmethod
    def writelines(self, __lines: Iterable[AnyStr]) -> None: ...
    @abstractmethod
    def __next__(self) -> AnyStr: ...
    @abstractmethod
    def __iter__(self) -> Iterator[AnyStr]: ...
    @abstractmethod
    def __enter__(self) -> IO[AnyStr]: ...
    @abstractmethod
    def __exit__(
        self, __t: Type[BaseException] | None, __value: BaseException | None, __traceback: TracebackType | None
    ) -> None: ...

class BinaryIO(IO[bytes]):
    @abstractmethod
    def __enter__(self) -> BinaryIO: ...

class TextIO(IO[str]):
    # TODO use abstractproperty
    @property
    def buffer(self) -> BinaryIO: ...
    @property
    def encoding(self) -> str: ...
    @property
    def errors(self) -> str | None: ...
    @property
    def line_buffering(self) -> int: ...  # int on PyPy, bool on CPython
    @property
    def newlines(self) -> Any: ...  # None, str or tuple
    @abstractmethod
    def __enter__(self) -> TextIO: ...

class ByteString(Sequence[int], metaclass=ABCMeta): ...

@_final
class Match(Generic[AnyStr]):
    @property
    def pos(self) -> int: ...
    @property
    def endpos(self) -> int: ...
    @property
    def lastindex(self) -> int | None: ...
    @property
    def lastgroup(self) -> str | None: ...
    @property
    def string(self) -> AnyStr: ...

    # The regular expression object whose match() or search() method produced
    # this match instance.
    @property
    def re(self) -> Pattern[AnyStr]: ...
    def expand(self, template: AnyStr) -> AnyStr: ...
    # group() returns "AnyStr" or "AnyStr | None", depending on the pattern.
    @overload
    def group(self, __group: _Literal[0] = ...) -> AnyStr: ...
    @overload
    def group(self, __group: str | int) -> AnyStr | Any: ...
    @overload
    def group(self, __group1: str | int, __group2: str | int, *groups: str | int) -> tuple[AnyStr | Any, ...]: ...
    # Each item of groups()'s return tuple is either "AnyStr" or
    # "AnyStr | None", depending on the pattern.
    @overload
    def groups(self) -> tuple[AnyStr | Any, ...]: ...
    @overload
    def groups(self, default: _T) -> tuple[AnyStr | _T, ...]: ...
    # Each value in groupdict()'s return dict is either "AnyStr" or
    # "AnyStr | None", depending on the pattern.
    @overload
    def groupdict(self) -> dict[str, AnyStr | Any]: ...
    @overload
    def groupdict(self, default: _T) -> dict[str, AnyStr | _T]: ...
    def start(self, __group: int | str = ...) -> int: ...
    def end(self, __group: int | str = ...) -> int: ...
    def span(self, __group: int | str = ...) -> tuple[int, int]: ...
    @property
    def regs(self) -> tuple[tuple[int, int], ...]: ...  # undocumented
    # __getitem__() returns "AnyStr" or "AnyStr | None", depending on the pattern.
    @overload
    def __getitem__(self, __key: _Literal[0]) -> AnyStr: ...
    @overload
    def __getitem__(self, __key: int | str) -> AnyStr | Any: ...
    def __copy__(self) -> Match[AnyStr]: ...
    def __deepcopy__(self, __memo: Any) -> Match[AnyStr]: ...
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

@_final
class Pattern(Generic[AnyStr]):
    @property
    def flags(self) -> int: ...
    @property
    def groupindex(self) -> Mapping[str, int]: ...
    @property
    def groups(self) -> int: ...
    @property
    def pattern(self) -> AnyStr: ...
    def search(self, string: AnyStr, pos: int = ..., endpos: int = ...) -> Match[AnyStr] | None: ...
    def match(self, string: AnyStr, pos: int = ..., endpos: int = ...) -> Match[AnyStr] | None: ...
    def fullmatch(self, string: AnyStr, pos: int = ..., endpos: int = ...) -> Match[AnyStr] | None: ...
    def split(self, string: AnyStr, maxsplit: int = ...) -> list[AnyStr | Any]: ...
    def findall(self, string: AnyStr, pos: int = ..., endpos: int = ...) -> list[Any]: ...
    def finditer(self, string: AnyStr, pos: int = ..., endpos: int = ...) -> Iterator[Match[AnyStr]]: ...
    @overload
    def sub(self, repl: AnyStr, string: AnyStr, count: int = ...) -> AnyStr: ...
    @overload
    def sub(self, repl: Callable[[Match[AnyStr]], AnyStr], string: AnyStr, count: int = ...) -> AnyStr: ...
    @overload
    def subn(self, repl: AnyStr, string: AnyStr, count: int = ...) -> tuple[AnyStr, int]: ...
    @overload
    def subn(self, repl: Callable[[Match[AnyStr]], AnyStr], string: AnyStr, count: int = ...) -> tuple[AnyStr, int]: ...
    def __copy__(self) -> Pattern[AnyStr]: ...
    def __deepcopy__(self, __memo: Any) -> Pattern[AnyStr]: ...
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

# Functions

if sys.version_info >= (3, 7):
    _get_type_hints_obj_allowed_types = (
        object
        | Callable[..., Any]
        | FunctionType
        | BuiltinFunctionType
        | MethodType
        | ModuleType
        | WrapperDescriptorType
        | MethodWrapperType
        | MethodDescriptorType
    )
else:
    _get_type_hints_obj_allowed_types = object | Callable[..., Any] | FunctionType | BuiltinFunctionType | MethodType | ModuleType

if sys.version_info >= (3, 9):
    def get_type_hints(
        obj: _get_type_hints_obj_allowed_types,
        globalns: dict[str, Any] | None = ...,
        localns: dict[str, Any] | None = ...,
        include_extras: bool = ...,
    ) -> dict[str, Any]: ...

else:
    def get_type_hints(
        obj: _get_type_hints_obj_allowed_types, globalns: dict[str, Any] | None = ..., localns: dict[str, Any] | None = ...
    ) -> dict[str, Any]: ...

if sys.version_info >= (3, 8):
    def get_origin(tp: Any) -> Any | None: ...
    def get_args(tp: Any) -> tuple[Any, ...]: ...

@overload
def cast(typ: Type[_T], val: Any) -> _T: ...
@overload
def cast(typ: str, val: Any) -> Any: ...
@overload
def cast(typ: object, val: Any) -> Any: ...

if sys.version_info >= (3, 11):
    def reveal_type(__obj: _T) -> _T: ...
    def assert_never(__arg: Never) -> Never: ...
    def assert_type(__val: _T, __typ: Any) -> _T: ...

# Type constructors

class NamedTuple(tuple[Any, ...]):
    if sys.version_info < (3, 8):
        _field_types: collections.OrderedDict[str, type]
    elif sys.version_info < (3, 9):
        _field_types: dict[str, type]
    _field_defaults: dict[str, Any]
    _fields: tuple[str, ...]
    _source: str
    @overload
    def __init__(self, typename: str, fields: Iterable[tuple[str, Any]] = ...) -> None: ...
    @overload
    def __init__(self, typename: str, fields: None = ..., **kwargs: Any) -> None: ...
    @classmethod
    def _make(cls: Type[_T], iterable: Iterable[Any]) -> _T: ...
    if sys.version_info >= (3, 8):
        def _asdict(self) -> dict[str, Any]: ...
    else:
        def _asdict(self) -> collections.OrderedDict[str, Any]: ...

    def _replace(self: TypeshedSelf, **kwargs: Any) -> TypeshedSelf: ...

# Internal mypy fallback type for all typed dicts (does not exist at runtime)
class _TypedDict(Mapping[str, object], metaclass=ABCMeta):
    def copy(self: TypeshedSelf) -> TypeshedSelf: ...
    # Using NoReturn so that only calls using mypy plugin hook that specialize the signature
    # can go through.
    def setdefault(self, k: NoReturn, default: object) -> object: ...
    # Mypy plugin hook for 'pop' expects that 'default' has a type variable type.
    def pop(self, k: NoReturn, default: _T = ...) -> object: ...  # type: ignore
    def update(self: _T, __m: _T) -> None: ...
    def __delitem__(self, k: NoReturn) -> None: ...
    def items(self) -> ItemsView[str, object]: ...
    def keys(self) -> KeysView[str]: ...
    def values(self) -> ValuesView[object]: ...
    def __or__(self: TypeshedSelf, __value: TypeshedSelf) -> TypeshedSelf: ...
    def __ior__(self: TypeshedSelf, __value: TypeshedSelf) -> TypeshedSelf: ...

# This itself is only available during type checking
def type_check_only(func_or_cls: _F) -> _F: ...

if sys.version_info >= (3, 7):
    @_final
    class ForwardRef:
        __forward_arg__: str
        __forward_code__: CodeType
        __forward_evaluated__: bool
        __forward_value__: Any | None
        __forward_is_argument__: bool
        __forward_is_class__: bool
        __forward_module__: Any | None
        if sys.version_info >= (3, 9):
            # The module and is_class arguments were added in later Python 3.9 versions.
            def __init__(self, arg: str, is_argument: bool = ..., module: Any | None = ..., *, is_class: bool = ...) -> None: ...
        else:
            def __init__(self, arg: str, is_argument: bool = ...) -> None: ...

        def _evaluate(self, globalns: dict[str, Any] | None, localns: dict[str, Any] | None) -> Any | None: ...
        def __eq__(self, other: object) -> bool: ...
        def __hash__(self) -> int: ...
        if sys.version_info >= (3, 11):
            def __or__(self, other: Any) -> _SpecialForm: ...
            def __ror__(self, other: Any) -> _SpecialForm: ...

if sys.version_info >= (3, 10):
    def is_typeddict(tp: object) -> bool: ...



from collections.abc import Sequence

from ttkthemes.themed_style import ThemedStyle as ThemedStyle
from ttkthemes.themed_tk import ThemedTk as ThemedTk

# actually a list, but shouldn't be modified
THEMES: Sequence[str]



from typing import Any

from selenium.webdriver.remote.webdriver import WebDriver as RemoteWebDriver

from .remote_connection import ChromeRemoteConnection as ChromeRemoteConnection

class WebDriver(RemoteWebDriver):
    service: Any
    def __init__(
        self,
        executable_path: str = ...,
        port: int = ...,
        options: Any | None = ...,
        service_args: Any | None = ...,
        desired_capabilities: Any | None = ...,
        service_log_path: Any | None = ...,
        chrome_options: Any | None = ...,
        keep_alive: bool = ...,
    ) -> None: ...
    def launch_app(self, id): ...
    def get_network_conditions(self): ...
    def set_network_conditions(self, **network_conditions) -> None: ...
    def execute_cdp_cmd(self, cmd, cmd_args): ...
    def quit(self) -> None: ...
    def create_options(self): ...



from _typeshed import Self, SupportsItems
from collections.abc import Iterable, Mapping, Sequence
from typing import Any, NamedTuple

from ..util import immutabledict
from .interfaces import Dialect

# stub-only helper class
class _URLTuple(NamedTuple):
    drivername: str
    username: str | None
    password: str | object | None  # object that produces a password when called with str()
    host: str | None
    port: int | None
    database: str | None
    query: immutabledict[str, str | tuple[str, ...]]

_Query = Mapping[str, str | Sequence[str]] | Sequence[tuple[str, str | Sequence[str]]]

class URL(_URLTuple):
    @classmethod
    def create(
        cls,
        drivername: str,
        username: str | None = ...,
        password: str | object | None = ...,  # object that produces a password when called with str()
        host: str | None = ...,
        port: int | None = ...,
        database: str | None = ...,
        query: _Query | None = ...,
    ) -> URL: ...
    def set(
        self: Self,
        drivername: str | None = ...,
        username: str | None = ...,
        password: str | object | None = ...,
        host: str | None = ...,
        port: int | None = ...,
        database: str | None = ...,
        query: _Query | None = ...,
    ) -> Self: ...
    def update_query_string(self: Self, query_string: str, append: bool = ...) -> Self: ...
    def update_query_pairs(self: Self, key_value_pairs: Iterable[tuple[str, str]], append: bool = ...) -> Self: ...
    def update_query_dict(self: Self, query_parameters: SupportsItems[str, str | Sequence[str]], append: bool = ...) -> Self: ...
    def difference_update_query(self, names: Iterable[str]) -> URL: ...
    @property
    def normalized_query(self) -> immutabledict[str, tuple[str, ...]]: ...
    def __to_string__(self, hide_password: bool = ...) -> str: ...
    def render_as_string(self, hide_password: bool = ...) -> str: ...
    def __copy__(self: Self) -> Self: ...
    def __deepcopy__(self: Self, memo: object) -> Self: ...
    def __hash__(self) -> int: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def get_backend_name(self) -> str: ...
    def get_driver_name(self) -> str: ...
    def get_dialect(self) -> type[Dialect]: ...
    def translate_connect_args(self, names: list[str] | None = ..., **kw: str) -> dict[str, Any]: ...

def make_url(name_or_url: str | URL) -> URL: ...



"""Test errors.py."""

import io
import math
import tempfile
import time

from pytype import metrics

import unittest


class MetricsTest(unittest.TestCase):
  """Tests for metrics infrastructure and the Counter class."""

  def setUp(self):
    super().setUp()
    metrics._prepare_for_test()

  def test_name_collision(self):
    metrics.Counter("foo")
    self.assertRaises(ValueError, metrics.Counter, "foo")

  def test_valid_name(self):
    # Metrics allows the same names as python identifiers.
    metrics.Counter("abc")
    metrics.Counter("a_b_c")
    metrics.Counter("abc1")
    metrics.Counter("_abc")

  def test_invalid_name(self):
    self.assertRaises(ValueError, metrics.Counter, "")
    self.assertRaises(ValueError, metrics.Counter, "a-b")
    self.assertRaises(ValueError, metrics.Counter, "a b")
    self.assertRaises(ValueError, metrics.Counter, "123")

  def test_get_report(self):
    metrics.Counter("foo").inc(2)
    metrics.Counter("bar").inc(123)
    self.assertEqual("bar: 123\nfoo: 2\n", metrics.get_report())

  def test_counter(self):
    c = metrics.Counter("foo")
    self.assertEqual(0, c._total)
    self.assertEqual("foo: 0", str(c))
    c.inc()
    self.assertEqual(1, c._total)
    c.inc(6)
    self.assertEqual(7, c._total)
    c.inc(0)
    self.assertEqual(7, c._total)
    self.assertEqual("foo: 7", str(c))
    self.assertRaises(ValueError, c.inc, -1)

  def test_counter_disabled(self):
    metrics._prepare_for_test(enabled=False)
    c = metrics.Counter("foo")
    c.inc()
    self.assertEqual(0, c._total)

  def test_merge_from(self):
    # Create a counter, increment it, and dump it.
    c1 = metrics.Counter("foo")
    c1.inc(1)
    dump = io.StringIO("")
    metrics.dump_all([c1], dump)
    # Reset metrics, merge from dump, which will create a new metric.
    metrics._prepare_for_test()
    self.assertFalse(metrics._registered_metrics)
    dump.seek(0)
    metrics.merge_from_file(dump)
    m = metrics._registered_metrics["foo"]
    self.assertEqual(1, m._total)
    # Merge again, this time it will merge data into the existing metric.
    dump.seek(0)
    metrics.merge_from_file(dump)
    self.assertEqual(2, m._total)
    # It's an error to merge an incompatible type.
    metrics._prepare_for_test()
    _ = metrics.MapCounter("foo")
    dump.seek(0)
    self.assertRaises(TypeError, metrics.merge_from_file, dump)

  def test_get_metric(self):
    c1 = metrics.get_metric("foo", metrics.Counter)
    self.assertIsInstance(c1, metrics.Counter)
    c2 = metrics.get_metric("foo", metrics.Counter)
    self.assertIs(c1, c2)


class ReentrantStopWatchTest(unittest.TestCase):

  def test_reentrant_stop_watch(self):
    c = metrics.ReentrantStopWatch("watch1")
    with c:
      with c:
        time.sleep(0.01)
    self.assertGreater(c._time, 0)

  def test_reentrant_stop_watch_merge(self):
    c = metrics.ReentrantStopWatch("watch2")
    d = metrics.ReentrantStopWatch("watch3")
    with c:
      with c:
        time.sleep(0.0025)

    with d:
      with d:
        time.sleep(0.0001)

    t1 = c._time
    t2 = d._time
    d._merge(c)
    self.assertLess(abs(t1+t2-d._time), 0.000001)


class StopWatchTest(unittest.TestCase):
  """Tests for StopWatch."""

  def setUp(self):
    super().setUp()
    metrics._prepare_for_test()

  def test_stopwatch(self):
    c = metrics.StopWatch("foo")
    with c:
      pass
    self.assertGreaterEqual(c._total, 0)

  def test_merge(self):
    c1 = metrics.StopWatch("foo")
    c2 = metrics.StopWatch("bar")
    with c1:
      pass
    with c2:
      pass
    t1 = c1._total
    t2 = c2._total
    c1._merge(c2)
    t3 = c1._total
    self.assertGreaterEqual(t3, t1)
    self.assertGreaterEqual(t3, t2)

  def test_summary(self):
    c1 = metrics.StopWatch("foo")
    with c1:
      pass
    self.assertIsInstance(c1._summary(), str)
    self.assertIsInstance(str(c1), str)


class MapCounterTest(unittest.TestCase):
  """Tests for MapCounter."""

  def setUp(self):
    super().setUp()
    metrics._prepare_for_test()

  def test_enabled(self):
    c = metrics.MapCounter("foo")
    # Check contents of an empty map.
    self.assertEqual(0, c._total)
    # Increment a few values and check again.
    c.inc("x")
    c.inc("y", 2)
    c.inc("x", 5)
    self.assertEqual(8, c._total)
    self.assertDictEqual(dict(x=6, y=2), c._counts)
    self.assertEqual("foo: 8 {x=6, y=2}", str(c))

  def test_disabled(self):
    metrics._prepare_for_test(enabled=False)
    c = metrics.MapCounter("foo")
    c.inc("x")
    self.assertEqual(0, c._total)

  def test_merge(self):
    c = metrics.MapCounter("foo")
    c.inc("x")
    c.inc("y", 2)
    # Cheat a little by merging a counter with a different name.
    other = metrics.MapCounter("other")
    other.inc("x")
    other.inc("z")
    c._merge(other)
    # Check merged contents.
    self.assertEqual(5, c._total)
    self.assertDictEqual(dict(x=2, y=2, z=1), c._counts)


class DistributionTest(unittest.TestCase):
  """Tests for Distribution."""

  def setUp(self):
    super().setUp()
    metrics._prepare_for_test()

  def test_accumulation(self):
    d = metrics.Distribution("foo")
    # Check contents of an empty distribution.
    self.assertEqual(0, d._count)
    self.assertEqual(0, d._total)
    self.assertIsNone(d._min)
    self.assertIsNone(d._max)
    self.assertIsNone(d._mean())
    self.assertIsNone(d._stdev())
    # Add some values.
    d.add(3)
    d.add(2)
    d.add(5)
    # Check the final contents.
    self.assertEqual(3, d._count)
    self.assertEqual(10, d._total)
    self.assertEqual(2, d._min)
    self.assertEqual(5, d._max)
    self.assertAlmostEqual(10.0 / 3, d._mean())
    # Stddev should be sqrt(14/9).
    self.assertAlmostEqual(math.sqrt(14.0 / 9), d._stdev())

  def test_summary(self):
    d = metrics.Distribution("foo")
    self.assertEqual(
        "foo: total=0.0, count=0, min=None, max=None, mean=None, stdev=None",
        str(d))
    # This test is delicate because it is checking the string output of
    # floating point calculations.  This specific data set was chosen because
    # the number of samples is a power of two (thus the division is exact) and
    # the variance is a natural square (thus the sqrt() is exact).
    d.add(1)
    d.add(5)
    self.assertEqual(
        "foo: total=6.0, count=2, min=1, max=5, mean=3.0, stdev=2.0",
        str(d))

  def test_disabled(self):
    metrics._prepare_for_test(enabled=False)
    d = metrics.Distribution("foo")
    d.add(123)
    self.assertEqual(0, d._count)

  def test_merge(self):
    d = metrics.Distribution("foo")
    # Merge two empty metrics together.
    other = metrics.Distribution("d_empty")
    d._merge(other)
    self.assertEqual(0, d._count)
    self.assertEqual(0, d._total)
    self.assertEqual(0, d._squared)
    self.assertEqual(None, d._min)
    self.assertEqual(None, d._max)
    # Merge into an empty metric (verifies the case where min/max must be
    # copied directly from the merged metric).
    other = metrics.Distribution("d2")
    other.add(10)
    other.add(20)
    d._merge(other)
    self.assertEqual(2, d._count)
    self.assertEqual(30, d._total)
    self.assertEqual(500, d._squared)
    self.assertEqual(10, d._min)
    self.assertEqual(20, d._max)
    # Merge into an existing metric resulting in a new min.
    other = metrics.Distribution("d3")
    other.add(5)
    d._merge(other)
    self.assertEqual(3, d._count)
    self.assertEqual(35, d._total)
    self.assertEqual(525, d._squared)
    self.assertEqual(5, d._min)
    self.assertEqual(20, d._max)
    # Merge into an existing metric resulting in a new max.
    other = metrics.Distribution("d4")
    other.add(30)
    d._merge(other)
    self.assertEqual(4, d._count)
    self.assertEqual(65, d._total)
    self.assertEqual(1425, d._squared)
    self.assertEqual(5, d._min)
    self.assertEqual(30, d._max)
    # Merge an empty metric (slopppy min/max code would fail).
    other = metrics.Distribution("d5")
    d._merge(other)
    self.assertEqual(4, d._count)
    self.assertEqual(65, d._total)
    self.assertEqual(1425, d._squared)
    self.assertEqual(5, d._min)
    self.assertEqual(30, d._max)


class MetricsContextTest(unittest.TestCase):
  """Tests for MetricsContext."""

  def setUp(self):
    super().setUp()
    metrics._prepare_for_test(False)
    self._counter = metrics.Counter("foo")

  def test_enabled(self):
    with tempfile.NamedTemporaryFile() as out:
      out.close()
      with metrics.MetricsContext(out.name):
        self._counter.inc()
      self.assertEqual(1, self._counter._total)
      with open(out.name) as f:
        dumped = metrics.load_all(f)
        self.assertEqual(len(dumped), 1)
        self.assertEqual("foo", dumped[0].name)
        self.assertEqual("foo: 1", str(dumped[0]))

  def test_disabled(self):
    with metrics.MetricsContext(""):
      self._counter.inc()
    self.assertEqual(0, self._counter._total)


if __name__ == "__main__":
  unittest.main()



from _typeshed import Self
from typing import Any

from ...util import memoized_property
from .base import ReversibleProxy, StartableContext

class AsyncSession(ReversibleProxy):
    dispatch: Any
    bind: Any
    binds: Any
    sync_session_class: Any
    sync_session: Any
    def __init__(self, bind: Any | None = ..., binds: Any | None = ..., sync_session_class: Any | None = ..., **kw) -> None: ...
    async def refresh(self, instance, attribute_names: Any | None = ..., with_for_update: Any | None = ...): ...
    async def run_sync(self, fn, *arg, **kw): ...
    async def execute(
        self, statement, params: Any | None = ..., execution_options=..., bind_arguments: Any | None = ..., **kw
    ): ...
    async def scalar(
        self, statement, params: Any | None = ..., execution_options=..., bind_arguments: Any | None = ..., **kw
    ): ...
    async def scalars(
        self, statement, params: Any | None = ..., execution_options=..., bind_arguments: Any | None = ..., **kw
    ): ...
    async def get(
        self,
        entity,
        ident,
        options: Any | None = ...,
        populate_existing: bool = ...,
        with_for_update: Any | None = ...,
        identity_token: Any | None = ...,
    ): ...
    async def stream(
        self, statement, params: Any | None = ..., execution_options=..., bind_arguments: Any | None = ..., **kw
    ): ...
    async def stream_scalars(
        self, statement, params: Any | None = ..., execution_options=..., bind_arguments: Any | None = ..., **kw
    ): ...
    async def delete(self, instance): ...
    async def merge(self, instance, load: bool = ..., options: Any | None = ...): ...
    async def flush(self, objects: Any | None = ...) -> None: ...
    def get_transaction(self): ...
    def get_nested_transaction(self): ...
    def get_bind(self, mapper: Any | None = ..., clause: Any | None = ..., bind: Any | None = ..., **kw): ...
    async def connection(self, **kw): ...
    def begin(self, **kw): ...
    def begin_nested(self, **kw): ...
    async def rollback(self): ...
    async def commit(self): ...
    async def close(self): ...
    @classmethod
    async def close_all(cls): ...
    async def __aenter__(self: Self) -> Self: ...
    async def __aexit__(self, type_, value, traceback) -> None: ...
    # proxied from Session
    identity_map: Any
    autoflush: Any
    @classmethod
    def identity_key(cls, *args, **kwargs): ...
    @classmethod
    def object_session(cls, instance): ...
    def __contains__(self, instance): ...
    def __iter__(self): ...
    def add(self, instance, _warn: bool = ...) -> None: ...
    def add_all(self, instances) -> None: ...
    def expire(self, instance, attribute_names: Any | None = ...) -> None: ...
    def expire_all(self) -> None: ...
    def expunge(self, instance) -> None: ...
    def expunge_all(self) -> None: ...
    def is_modified(self, instance, include_collections: bool = ...): ...
    def in_transaction(self): ...
    def in_nested_transaction(self): ...
    @property
    def no_autoflush(self) -> None: ...
    @property
    def is_active(self): ...
    @property
    def dirty(self): ...
    @property
    def deleted(self): ...
    @property
    def new(self): ...
    @memoized_property
    def info(self): ...

class _AsyncSessionContextManager:
    async_session: Any
    def __init__(self, async_session) -> None: ...
    trans: Any
    async def __aenter__(self): ...
    async def __aexit__(self, type_, value, traceback) -> None: ...

class AsyncSessionTransaction(ReversibleProxy, StartableContext):
    session: Any
    nested: Any
    sync_transaction: Any
    def __init__(self, session, nested: bool = ...) -> None: ...
    @property
    def is_active(self): ...
    async def rollback(self) -> None: ...
    async def commit(self) -> None: ...
    async def start(self, is_ctxmanager: bool = ...): ...
    async def __aexit__(self, type_, value, traceback) -> None: ...

def async_object_session(instance): ...
def async_session(session): ...



import sys

if sys.platform != "win32":
    class _Method: ...
    METHOD_CRYPT: _Method
    METHOD_MD5: _Method
    METHOD_SHA256: _Method
    METHOD_SHA512: _Method
    if sys.version_info >= (3, 7):
        METHOD_BLOWFISH: _Method

    methods: list[_Method]

    if sys.version_info >= (3, 7):
        def mksalt(method: _Method | None = ..., *, rounds: int | None = ...) -> str: ...
    else:
        def mksalt(method: _Method | None = ...) -> str: ...

    def crypt(word: str, salt: str | _Method | None = ...) -> str: ...



from .rfc6749.clients import (
    BackendApplicationClient as BackendApplicationClient,
    Client as Client,
    LegacyApplicationClient as LegacyApplicationClient,
    MobileApplicationClient as MobileApplicationClient,
    ServiceApplicationClient as ServiceApplicationClient,
    WebApplicationClient as WebApplicationClient,
)
from .rfc6749.endpoints import (
    AuthorizationEndpoint as AuthorizationEndpoint,
    BackendApplicationServer as BackendApplicationServer,
    IntrospectEndpoint as IntrospectEndpoint,
    LegacyApplicationServer as LegacyApplicationServer,
    MetadataEndpoint as MetadataEndpoint,
    MobileApplicationServer as MobileApplicationServer,
    ResourceEndpoint as ResourceEndpoint,
    RevocationEndpoint as RevocationEndpoint,
    Server as Server,
    TokenEndpoint as TokenEndpoint,
    WebApplicationServer as WebApplicationServer,
)
from .rfc6749.errors import (
    AccessDeniedError as AccessDeniedError,
    FatalClientError as FatalClientError,
    InsecureTransportError as InsecureTransportError,
    InvalidClientError as InvalidClientError,
    InvalidClientIdError as InvalidClientIdError,
    InvalidGrantError as InvalidGrantError,
    InvalidRedirectURIError as InvalidRedirectURIError,
    InvalidRequestError as InvalidRequestError,
    InvalidRequestFatalError as InvalidRequestFatalError,
    InvalidScopeError as InvalidScopeError,
    MismatchingRedirectURIError as MismatchingRedirectURIError,
    MismatchingStateError as MismatchingStateError,
    MissingClientIdError as MissingClientIdError,
    MissingCodeError as MissingCodeError,
    MissingRedirectURIError as MissingRedirectURIError,
    MissingResponseTypeError as MissingResponseTypeError,
    MissingTokenError as MissingTokenError,
    MissingTokenTypeError as MissingTokenTypeError,
    OAuth2Error as OAuth2Error,
    ServerError as ServerError,
    TemporarilyUnavailableError as TemporarilyUnavailableError,
    TokenExpiredError as TokenExpiredError,
    UnauthorizedClientError as UnauthorizedClientError,
    UnsupportedGrantTypeError as UnsupportedGrantTypeError,
    UnsupportedResponseTypeError as UnsupportedResponseTypeError,
    UnsupportedTokenTypeError as UnsupportedTokenTypeError,
)
from .rfc6749.grant_types import (
    AuthorizationCodeGrant as AuthorizationCodeGrant,
    ClientCredentialsGrant as ClientCredentialsGrant,
    ImplicitGrant as ImplicitGrant,
    RefreshTokenGrant as RefreshTokenGrant,
    ResourceOwnerPasswordCredentialsGrant as ResourceOwnerPasswordCredentialsGrant,
)
from .rfc6749.request_validator import RequestValidator as RequestValidator
from .rfc6749.tokens import BearerToken as BearerToken, OAuth2Token as OAuth2Token
from .rfc6749.utils import is_secure_transport as is_secure_transport



from typing import Any

AUTH_HEADER: str
URI_QUERY: str
BODY: str
FORM_ENC_HEADERS: Any

class Client:
    refresh_token_key: str
    client_id: Any
    default_token_placement: Any
    token_type: Any
    access_token: Any
    refresh_token: Any
    mac_key: Any
    mac_algorithm: Any
    token: Any
    scope: Any
    state_generator: Any
    state: Any
    redirect_url: Any
    code: Any
    expires_in: Any
    def __init__(
        self,
        client_id,
        default_token_placement=...,
        token_type: str = ...,
        access_token: Any | None = ...,
        refresh_token: Any | None = ...,
        mac_key: Any | None = ...,
        mac_algorithm: Any | None = ...,
        token: Any | None = ...,
        scope: Any | None = ...,
        state: Any | None = ...,
        redirect_url: Any | None = ...,
        state_generator=...,
        **kwargs,
    ) -> None: ...
    @property
    def token_types(self): ...
    def prepare_request_uri(self, *args, **kwargs) -> None: ...
    def prepare_request_body(self, *args, **kwargs) -> None: ...
    def parse_request_uri_response(self, *args, **kwargs) -> None: ...
    def add_token(
        self,
        uri,
        http_method: str = ...,
        body: Any | None = ...,
        headers: Any | None = ...,
        token_placement: Any | None = ...,
        **kwargs,
    ): ...
    def prepare_authorization_request(
        self, authorization_url, state: Any | None = ..., redirect_url: Any | None = ..., scope: Any | None = ..., **kwargs
    ): ...
    def prepare_token_request(
        self,
        token_url,
        authorization_response: Any | None = ...,
        redirect_url: Any | None = ...,
        state: Any | None = ...,
        body: str = ...,
        **kwargs,
    ): ...
    def prepare_refresh_token_request(
        self, token_url, refresh_token: Any | None = ..., body: str = ..., scope: Any | None = ..., **kwargs
    ): ...
    def prepare_token_revocation_request(
        self, revocation_url, token, token_type_hint: str = ..., body: str = ..., callback: Any | None = ..., **kwargs
    ): ...
    def parse_request_body_response(self, body, scope: Any | None = ..., **kwargs): ...
    def prepare_refresh_body(self, body: str = ..., refresh_token: Any | None = ..., scope: Any | None = ..., **kwargs): ...
    def populate_code_attributes(self, response) -> None: ...
    def populate_token_attributes(self, response) -> None: ...



from typing import Any

class Log:
    level: Any
    def __init__(self) -> None: ...
    def to_capabilities(self): ...

class Options:
    KEY: str
    log: Any
    def __init__(self) -> None: ...
    @property
    def binary(self): ...
    @binary.setter
    def binary(self, new_binary) -> None: ...
    @property
    def binary_location(self): ...
    @binary_location.setter
    def binary_location(self, value) -> None: ...
    @property
    def accept_insecure_certs(self): ...
    @accept_insecure_certs.setter
    def accept_insecure_certs(self, value) -> None: ...
    @property
    def capabilities(self): ...
    def set_capability(self, name, value) -> None: ...
    @property
    def preferences(self): ...
    def set_preference(self, name, value) -> None: ...
    @property
    def proxy(self): ...
    @proxy.setter
    def proxy(self, value) -> None: ...
    @property
    def profile(self): ...
    @profile.setter
    def profile(self, new_profile) -> None: ...
    @property
    def arguments(self): ...
    def add_argument(self, argument) -> None: ...
    @property
    def headless(self): ...
    @headless.setter
    def headless(self, value) -> None: ...
    def set_headless(self, headless: bool = ...) -> None: ...
    def to_capabilities(self): ...



import datetime
from typing import IO, Any, Text, TypeVar

from ..relativedelta import relativedelta
from ._common import _tzinfo as _tzinfo, enfold as enfold, tzname_in_python2 as tzname_in_python2, tzrangebase as tzrangebase

_FileObj = str | Text | IO[str] | IO[Text]
_DT = TypeVar("_DT", bound=datetime.datetime)

ZERO: datetime.timedelta
EPOCH: datetime.datetime
EPOCHORDINAL: int

class tzutc(datetime.tzinfo):
    def utcoffset(self, dt: datetime.datetime | None) -> datetime.timedelta | None: ...
    def dst(self, dt: datetime.datetime | None) -> datetime.timedelta | None: ...
    def tzname(self, dt: datetime.datetime | None) -> str: ...
    def is_ambiguous(self, dt: datetime.datetime | None) -> bool: ...
    def fromutc(self, dt: _DT) -> _DT: ...
    def __eq__(self, other): ...
    __hash__: Any
    def __ne__(self, other): ...
    __reduce__: Any

class tzoffset(datetime.tzinfo):
    def __init__(self, name, offset) -> None: ...
    def utcoffset(self, dt: datetime.datetime | None) -> datetime.timedelta | None: ...
    def dst(self, dt: datetime.datetime | None) -> datetime.timedelta | None: ...
    def is_ambiguous(self, dt: datetime.datetime | None) -> bool: ...
    def tzname(self, dt: datetime.datetime | None) -> str: ...
    def fromutc(self, dt: _DT) -> _DT: ...
    def __eq__(self, other): ...
    __hash__: Any
    def __ne__(self, other): ...
    __reduce__: Any
    @classmethod
    def instance(cls, name, offset) -> tzoffset: ...

class tzlocal(_tzinfo):
    def __init__(self) -> None: ...
    def utcoffset(self, dt: datetime.datetime | None) -> datetime.timedelta | None: ...
    def dst(self, dt: datetime.datetime | None) -> datetime.timedelta | None: ...
    def tzname(self, dt: datetime.datetime | None) -> str: ...
    def is_ambiguous(self, dt: datetime.datetime | None) -> bool: ...
    def __eq__(self, other): ...
    __hash__: Any
    def __ne__(self, other): ...
    __reduce__: Any

class _ttinfo:
    def __init__(self) -> None: ...
    def __eq__(self, other): ...
    __hash__: Any
    def __ne__(self, other): ...

class tzfile(_tzinfo):
    def __init__(self, fileobj: _FileObj, filename: Text | None = ...) -> None: ...
    def is_ambiguous(self, dt: datetime.datetime | None, idx: int | None = ...) -> bool: ...
    def utcoffset(self, dt: datetime.datetime | None) -> datetime.timedelta | None: ...
    def dst(self, dt: datetime.datetime | None) -> datetime.timedelta | None: ...
    def tzname(self, dt: datetime.datetime | None) -> str: ...
    def __eq__(self, other): ...
    __hash__: Any
    def __ne__(self, other): ...
    def __reduce__(self): ...
    def __reduce_ex__(self, protocol): ...

class tzrange(tzrangebase):
    hasdst: bool
    def __init__(
        self,
        stdabbr: Text,
        stdoffset: int | datetime.timedelta | None = ...,
        dstabbr: Text | None = ...,
        dstoffset: int | datetime.timedelta | None = ...,
        start: relativedelta | None = ...,
        end: relativedelta | None = ...,
    ) -> None: ...
    def transitions(self, year: int) -> tuple[datetime.datetime, datetime.datetime]: ...
    def __eq__(self, other): ...

class tzstr(tzrange):
    hasdst: bool
    def __init__(self, s: bytes | _FileObj, posix_offset: bool = ...) -> None: ...
    @classmethod
    def instance(cls, name, offset) -> tzoffset: ...

class tzical:
    def __init__(self, fileobj: _FileObj) -> None: ...
    def keys(self): ...
    def get(self, tzid: Any | None = ...): ...

TZFILES: list[str]
TZPATHS: list[str]

def datetime_exists(dt: datetime.datetime, tz: datetime.tzinfo | None = ...) -> bool: ...
def datetime_ambiguous(dt: datetime.datetime, tz: datetime.tzinfo | None = ...) -> bool: ...
def resolve_imaginary(dt: datetime.datetime) -> datetime.datetime: ...

class _GetTZ:
    def __call__(self, name: Text | None = ...) -> datetime.tzinfo | None: ...
    def nocache(self, name: Text | None) -> datetime.tzinfo | None: ...

gettz: _GetTZ



from selenium.webdriver.remote.webelement import WebElement as RemoteWebElement

class FirefoxWebElement(RemoteWebElement):
    @property
    def anonymous_children(self): ...
    def find_anonymous_element_by_attribute(self, name, value): ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class GroupShapeProperties(Serialisable):
    tagname: str
    bwMode: Any
    xfrm: Any
    scene3d: Any
    extLst: Any
    def __init__(
        self, bwMode: Any | None = ..., xfrm: Any | None = ..., scene3d: Any | None = ..., extLst: Any | None = ...
    ) -> None: ...

class GroupLocking(Serialisable):
    tagname: str
    namespace: Any
    noGrp: Any
    noUngrp: Any
    noSelect: Any
    noRot: Any
    noChangeAspect: Any
    noMove: Any
    noResize: Any
    noChangeArrowheads: Any
    noEditPoints: Any
    noAdjustHandles: Any
    noChangeShapeType: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self,
        noGrp: Any | None = ...,
        noUngrp: Any | None = ...,
        noSelect: Any | None = ...,
        noRot: Any | None = ...,
        noChangeAspect: Any | None = ...,
        noChangeArrowheads: Any | None = ...,
        noMove: Any | None = ...,
        noResize: Any | None = ...,
        noEditPoints: Any | None = ...,
        noAdjustHandles: Any | None = ...,
        noChangeShapeType: Any | None = ...,
        extLst: Any | None = ...,
    ) -> None: ...

class NonVisualGroupDrawingShapeProps(Serialisable):
    tagname: str
    grpSpLocks: Any
    extLst: Any
    __elements__: Any
    def __init__(self, grpSpLocks: Any | None = ..., extLst: Any | None = ...) -> None: ...

class NonVisualDrawingShapeProps(Serialisable):
    tagname: str
    spLocks: Any
    txBax: Any
    extLst: Any
    __elements__: Any
    txBox: Any
    def __init__(self, spLocks: Any | None = ..., txBox: Any | None = ..., extLst: Any | None = ...) -> None: ...

class NonVisualDrawingProps(Serialisable):
    tagname: str
    id: Any
    name: Any
    descr: Any
    hidden: Any
    title: Any
    hlinkClick: Any
    hlinkHover: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self,
        id: Any | None = ...,
        name: Any | None = ...,
        descr: Any | None = ...,
        hidden: Any | None = ...,
        title: Any | None = ...,
        hlinkClick: Any | None = ...,
        hlinkHover: Any | None = ...,
        extLst: Any | None = ...,
    ) -> None: ...

class NonVisualGroupShape(Serialisable):
    tagname: str
    cNvPr: Any
    cNvGrpSpPr: Any
    __elements__: Any
    def __init__(self, cNvPr: Any | None = ..., cNvGrpSpPr: Any | None = ...) -> None: ...



"""Preprocess source code before compilation."""

import ast


# pylint: disable=invalid-name
class CollectAnnotationLines(ast.NodeVisitor):
  """Collect line numbers of annotations to augment."""

  def __init__(self):
    self.annotation_lines = []
    self.in_function = False

  def visit_AnnAssign(self, node):
    if self.in_function and node.value is None:
      self.annotation_lines.append(node.end_lineno - 1)  # change to 0-based

  def visit_FunctionDef(self, node):
    self.in_function = True
    for n in node.body:
      self.visit(n)
    self.in_function = False
# pylint: enable=invalid-name


def augment_annotations(src):
  """Add an assignment to bare variable annotations."""
  try:
    tree = ast.parse(src)
  except SyntaxError:
    # Let the compiler catch and report this later.
    return src
  visitor = CollectAnnotationLines()
  visitor.visit(tree)
  if visitor.annotation_lines:
    lines = src.split("\n")
    for i in visitor.annotation_lines:
      # Preserve comments, as they may be pytype directives. We don't bother to
      # keep the formatting, since users never see the transformed source code.
      line, mark, comment = lines[i].partition("#")
      lines[i] = line + " = ..." + mark + comment
    src = "\n".join(lines)
  return src



from typing import Any

from braintree.search import Search as Search

class DisputeSearch:
    amount_disputed: Any
    amount_won: Any
    case_number: Any
    chargeback_protection_level: Any
    customer_id: Any
    disbursement_date: Any
    effective_date: Any
    id: Any
    kind: Any
    merchant_account_id: Any
    reason: Any
    reason_code: Any
    received_date: Any
    reference_number: Any
    reply_by_date: Any
    status: Any
    transaction_id: Any
    transaction_source: Any



from typing import Any

class Alert:
    driver: Any
    def __init__(self, driver) -> None: ...
    @property
    def text(self): ...
    def dismiss(self) -> None: ...
    def accept(self) -> None: ...
    def send_keys(self, keysToSend) -> None: ...



import sys
from typing import IO

_FD = int | IO[str]

if sys.platform != "win32":
    # XXX: Undocumented integer constants
    IFLAG: int
    OFLAG: int
    CFLAG: int
    LFLAG: int
    ISPEED: int
    OSPEED: int
    CC: int
    def setraw(fd: _FD, when: int = ...) -> None: ...
    def setcbreak(fd: _FD, when: int = ...) -> None: ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class LegendEntry(Serialisable):
    tagname: str
    idx: Any
    delete: Any
    txPr: Any
    extLst: Any
    __elements__: Any
    def __init__(self, idx: int = ..., delete: bool = ..., txPr: Any | None = ..., extLst: Any | None = ...) -> None: ...

class Legend(Serialisable):
    tagname: str
    legendPos: Any
    position: Any
    legendEntry: Any
    layout: Any
    overlay: Any
    spPr: Any
    graphicalProperties: Any
    txPr: Any
    textProperties: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self,
        legendPos: str = ...,
        legendEntry=...,
        layout: Any | None = ...,
        overlay: Any | None = ...,
        spPr: Any | None = ...,
        txPr: Any | None = ...,
        extLst: Any | None = ...,
    ) -> None: ...



def fingerprint128(__a: str) -> tuple[int, int]: ...
def fingerprint32(__a: str) -> int: ...
def fingerprint64(__a: str) -> int: ...
def hash128(__a: str) -> tuple[int, int]: ...
def hash128withseed(__a: str, __seed_low: int, __seed_high: int) -> tuple[int, int]: ...
def hash32(__a: str) -> int: ...
def hash32withseed(__a: str, __seed: int) -> int: ...
def hash64(__a: str) -> int: ...
def hash64withseed(__a: str, __seed: int) -> int: ...



"""Tests for traces.source."""

from pytype.tools.traces import source
import unittest


class _FakeOpcode:

  def __init__(self, name, line):
    self.name = name
    self.line = line


class _FakeTrace(source.AbstractTrace):
  """Fake trace class for testing."""


class AbstractTraceTest(unittest.TestCase):

  def test_instantiate(self):
    with self.assertRaises(TypeError):
      source.AbstractTrace(None, None, None)
    self.assertIsInstance(_FakeTrace(None, None, None), _FakeTrace)

  def test_repr(self):
    trace = _FakeTrace("LOAD_NAME", "x", (["t"],))
    print(repr(trace))  # smoke test


class CodeTest(unittest.TestCase):
  """Tests for source.Code."""

  def test_basic(self):
    raw_traces = [(_FakeOpcode("op1", 1), "symbol1", (["data1"],)),
                  (_FakeOpcode("op2", 4), "symbol2", (["data2"],)),
                  (_FakeOpcode("op3", 1), "symbol3", (["data3"],))]
    src = source.Code("text", raw_traces, _FakeTrace, "name")
    self.assertEqual(src.text, "text")
    self.assertEqual(src.filename, "name")
    self.assertCountEqual(src.traces, [1, 4])
    self.assertEqual(
        src.traces[1],
        [_FakeTrace("op1", "symbol1", (["data1"],)),
         _FakeTrace("op3", "symbol3", (["data3"],))])
    self.assertEqual(
        src.traces[4], [_FakeTrace("op2", "symbol2", (["data2"],))])

  def test_get_offset(self):
    src = source.Code("line1\nline2", [], _FakeTrace, "")
    self.assertEqual(src.get_offset(source.Location(2, 3)), 9)

  def test_get_offset_multibyte(self):
    # With single-byte characters
    src = source.Code("""
      # coding=utf-8
      line1 # a
      line2
    """, [], _FakeTrace, "")
    self.assertEqual(src.get_offset(source.Location(4, 3)), 41)

    # With a multibyte character the byte offset should change
    src = source.Code("""
      # coding=utf-8
      line1 # ツ
      line2
    """, [], _FakeTrace, "")
    self.assertEqual(src.get_offset(source.Location(4, 3)), 43)

  def test_line(self):
    src = source.Code("line1\nline2", [], _FakeTrace, "")
    self.assertEqual(src.line(2), "line2")

  def test_get_closest_line_range(self):
    src = source.Code("line1\nline2\nline3", [], _FakeTrace, "")
    self.assertCountEqual(src.get_closest_line_range(1, 3), [1, 2])
    self.assertCountEqual(src.get_closest_line_range(2, 5), [2, 3])

  def test_find_first_text(self):
    src = source.Code("line1\nline2\nline3", [], _FakeTrace, "")
    self.assertEqual(src.find_first_text(2, 5, "line"), source.Location(2, 0))
    self.assertIsNone(src.find_first_text(2, 5, "duck"))

  def test_next_non_comment_line(self):
    src = source.Code("line1\n# line2\nline3", [], _FakeTrace, "")
    self.assertEqual(src.next_non_comment_line(1), 3)
    self.assertIsNone(src.next_non_comment_line(3))

  def test_display_traces(self):
    raw_traces = [(_FakeOpcode("op1", 1), "symbol1", (["data1"],)),
                  (_FakeOpcode("op2", 3), "symbol2", (None,))]
    src = source.Code("line1\nline2\nline3", raw_traces, _FakeTrace, "")
    src.display_traces()  # smoke test


class GetAttrLocationTest(unittest.TestCase):

  def test_one_line(self):
    src = source.Code("foo.bar", [], _FakeTrace, "")
    self.assertEqual(src.get_attr_location("foo.bar", source.Location(1, 0)),
                     (source.Location(1, 4), 3))

  def test_value_dot(self):
    src = source.Code("foo.\nbar", [], _FakeTrace, "")
    self.assertEqual(src.get_attr_location("foo.bar", source.Location(1, 0)),
                     (source.Location(2, 0), 3))

  def test_dot_attr(self):
    src = source.Code("foo\n.bar", [], _FakeTrace, "")
    self.assertEqual(src.get_attr_location("foo.bar", source.Location(1, 0)),
                     (source.Location(2, 1), 3))

  def test_not_found(self):
    src = source.Code("foo.bar", [], _FakeTrace, "")
    self.assertEqual(src.get_attr_location("foo.baz", source.Location(1, 0)),
                     (source.Location(1, 0), 7))


if __name__ == "__main__":
  unittest.main()



from typing import Any

from braintree.util.crypto import Crypto as Crypto
from braintree.webhook_notification import WebhookNotification as WebhookNotification

class WebhookTestingGateway:
    gateway: Any
    config: Any
    def __init__(self, gateway) -> None: ...
    def sample_notification(self, kind, id, source_merchant_id: Any | None = ...): ...



"""Apply decorators to classes and functions."""

from typing import Iterable, List, Tuple

from pytype.pytd import base_visitor
from pytype.pytd import pytd
from pytype.pytd.codegen import function


class ValidateDecoratedClassVisitor(base_visitor.Visitor):
  """Apply class decorators."""

  def EnterClass(self, cls):
    validate_class(cls)


def _decorate_class(cls: pytd.Class, decorator: str) -> Tuple[pytd.Class, bool]:
  """Apply a single decorator to a class."""
  factory = _DECORATORS.get(decorator, None)
  if factory:
    return factory(cls), True
  else:
    # do nothing for unknown decorators
    return cls, False


def _validate_class(cls: pytd.Class, decorator: str) -> None:
  """Validate a single decorator for a class."""
  validator = _VALIDATORS.get(decorator, None)
  if validator:
    validator(cls)


def _decorator_names(cls: pytd.Class) -> List[str]:
  return [x.type.name for x in reversed(cls.decorators)]


def has_decorator(cls: pytd.Class, names):
  return bool(set(names) & set(_decorator_names(cls)))


def check_defaults(fields: Iterable[pytd.Constant], cls_name: str):
  """Check that a non-default field does not follow a default one."""
  default = None
  for c in fields:
    if c.value is not None:
      default = c.name
    elif default:
      raise TypeError(
          f"In class {cls_name}: "
          f"non-default argument {c.name} follows default argument {default}")


def check_class(cls: pytd.Class) -> None:
  fields = get_attributes(cls)
  check_defaults(fields, cls.name)


def add_init_from_fields(
    cls: pytd.Class,
    fields: Iterable[pytd.Constant]
) -> pytd.Class:
  check_defaults(fields, cls.name)
  init = function.generate_init(fields)
  methods = cls.methods + (init,)
  return cls.Replace(methods=methods)


def get_attributes(cls: pytd.Class):
  """Get class attributes, filtering out properties and ClassVars."""
  attributes = []
  for c in cls.constants:
    if isinstance(c.type, pytd.Annotated):
      if "'property'" not in c.type.annotations:
        c = c.Replace(type=c.type.base_type)
        attributes.append(c)
    elif c.name == "__doc__":
      # Filter docstrings out from the attribs list
      # (we emit them as `__doc__ : str` in pyi output)
      pass
    elif c.type.name == "typing.ClassVar":
      # We do not want classvars treated as attribs
      pass
    else:
      attributes.append(c)
  return tuple(attributes)


def add_generated_init(cls: pytd.Class) -> pytd.Class:
  # Do not override an __init__ from the pyi file
  if any(x.name == "__init__" for x in cls.methods):
    return cls
  fields = get_attributes(cls)
  return add_init_from_fields(cls, fields)


def process_class(cls: pytd.Class) -> Tuple[pytd.Class, bool]:
  """Apply all decorators to a class."""
  changed = False
  for decorator in _decorator_names(cls):
    cls, decorated = _decorate_class(cls, decorator)
    changed = changed or decorated
  return cls, changed


def validate_class(cls: pytd.Class) -> None:
  for decorator in _decorator_names(cls):
    _validate_class(cls, decorator)


# NOTE: For attrs, the resolved "real name" of the decorator in pyi files is
# attr._make.attrs; the aliases are added here in case the attrs stub files
# change to hide that implementation detail. We also add an implicit
# "auto_attribs=True" to @attr.s decorators in pyi files.

_DECORATORS = {
    "dataclasses.dataclass": add_generated_init,
    "attr.s": add_generated_init,
    "attr.attrs": add_generated_init,
    "attr._make.attrs": add_generated_init,
}


_VALIDATORS = {
    "dataclasses.dataclass": check_class,
    "attr.s": check_class,
    "attr.attrs": check_class,
    "attr._make.attrs": check_class,
}



from typing import Any

class FirefoxBinary:
    NO_FOCUS_LIBRARY_NAME: str
    command_line: Any
    def __init__(self, firefox_path: Any | None = ..., log_file: Any | None = ...) -> None: ...
    def add_command_line_options(self, *args) -> None: ...
    profile: Any
    def launch_browser(self, profile, timeout: int = ...) -> None: ...
    def kill(self) -> None: ...
    def which(self, fname): ...



from collections.abc import Iterable, Iterator
from typing import NamedTuple, Pattern

from . import exceptions as exceptions
from .snippets import findall_regex as findall_regex, split_by_regex as split_by_regex

class header(NamedTuple):
    index_path: str | None
    old_path: str
    old_version: str | None
    new_path: str
    new_version: str | None

class diffobj(NamedTuple):
    header: header | None
    changes: list[Change] | None
    text: str

class Change(NamedTuple):
    old: int | None
    new: int | None
    line: int | None
    hunk: int

file_timestamp_str: str
diffcmd_header: Pattern[str]
unified_header_index: Pattern[str]
unified_header_old_line: Pattern[str]
unified_header_new_line: Pattern[str]
unified_hunk_start: Pattern[str]
unified_change: Pattern[str]
context_header_old_line: Pattern[str]
context_header_new_line: Pattern[str]
context_hunk_start: Pattern[str]
context_hunk_old: Pattern[str]
context_hunk_new: Pattern[str]
context_change: Pattern[str]
ed_hunk_start: Pattern[str]
ed_hunk_end: Pattern[str]
rcs_ed_hunk_start: Pattern[str]
default_hunk_start: Pattern[str]
default_hunk_mid: Pattern[str]
default_change: Pattern[str]
git_diffcmd_header: Pattern[str]
git_header_index: Pattern[str]
git_header_old_line: Pattern[str]
git_header_new_line: Pattern[str]
git_header_file_mode: Pattern[str]
git_header_binary_file: Pattern[str]
bzr_header_index: Pattern[str]
bzr_header_old_line: Pattern[str]
bzr_header_new_line: Pattern[str]
svn_header_index: Pattern[str]
svn_header_timestamp_version: Pattern[str]
svn_header_timestamp: Pattern[str]
cvs_header_index: Pattern[str]
cvs_header_rcs: Pattern[str]
cvs_header_timestamp: Pattern[str]
cvs_header_timestamp_colon: Pattern[str]
old_cvs_diffcmd_header: Pattern[str]

def parse_patch(text: str | Iterable[str]) -> Iterator[diffobj]: ...
def parse_header(text: str | Iterable[str]) -> header | None: ...
def parse_scm_header(text: str | Iterable[str]) -> header | None: ...
def parse_diff_header(text: str | Iterable[str]) -> header | None: ...
def parse_diff(text: str | Iterable[str]) -> list[Change] | None: ...
def parse_git_header(text: str | Iterable[str]) -> header | None: ...
def parse_svn_header(text: str | Iterable[str]) -> header | None: ...
def parse_cvs_header(text: str | Iterable[str]) -> header | None: ...
def parse_diffcmd_header(text: str | Iterable[str]) -> header | None: ...
def parse_unified_header(text: str | Iterable[str]) -> header | None: ...
def parse_context_header(text: str | Iterable[str]) -> header | None: ...
def parse_default_diff(text: str | Iterable[str]) -> list[Change] | None: ...
def parse_unified_diff(text: str | Iterable[str]) -> list[Change] | None: ...
def parse_context_diff(text: str | Iterable[str]) -> list[Change] | None: ...
def parse_ed_diff(text: str | Iterable[str]) -> list[Change] | None: ...
def parse_rcs_ed_diff(text: str | Iterable[str]) -> list[Change] | None: ...



from typing import Any

from .behavior import Behavior

class Name:
    family: Any
    given: Any
    additional: Any
    prefix: Any
    suffix: Any
    def __init__(
        self, family: str = ..., given: str = ..., additional: str = ..., prefix: str = ..., suffix: str = ...
    ) -> None: ...
    @staticmethod
    def toString(val): ...
    def __eq__(self, other): ...

class Address:
    box: Any
    extended: Any
    street: Any
    city: Any
    region: Any
    code: Any
    country: Any
    def __init__(
        self,
        street: str = ...,
        city: str = ...,
        region: str = ...,
        code: str = ...,
        country: str = ...,
        box: str = ...,
        extended: str = ...,
    ) -> None: ...
    @staticmethod
    def toString(val, join_char: str = ...): ...
    lines: Any
    one_line: Any
    def __eq__(self, other): ...

class VCardTextBehavior(Behavior):
    allowGroup: bool
    base64string: str
    @classmethod
    def decode(cls, line) -> None: ...
    @classmethod
    def encode(cls, line) -> None: ...

class VCardBehavior(Behavior):
    allowGroup: bool
    defaultBehavior: Any

class VCard3_0(VCardBehavior):
    name: str
    description: str
    versionString: str
    isComponent: bool
    sortFirst: Any
    knownChildren: Any
    @classmethod
    def generateImplicitParameters(cls, obj) -> None: ...

class FN(VCardTextBehavior):
    name: str
    description: str

class Label(VCardTextBehavior):
    name: str
    description: str

wacky_apple_photo_serialize: bool
REALLY_LARGE: float

class Photo(VCardTextBehavior):
    name: str
    description: str
    @classmethod
    def valueRepr(cls, line): ...
    @classmethod
    def serialize(cls, obj, buf, lineLength, validate) -> None: ...  # type: ignore[override]

def toListOrString(string): ...
def splitFields(string): ...
def toList(stringOrList): ...
def serializeFields(obj, order: Any | None = ...): ...

NAME_ORDER: Any
ADDRESS_ORDER: Any

class NameBehavior(VCardBehavior):
    hasNative: bool
    @staticmethod
    def transformToNative(obj): ...
    @staticmethod
    def transformFromNative(obj): ...

class AddressBehavior(VCardBehavior):
    hasNative: bool
    @staticmethod
    def transformToNative(obj): ...
    @staticmethod
    def transformFromNative(obj): ...

class OrgBehavior(VCardBehavior):
    hasNative: bool
    @staticmethod
    def transformToNative(obj): ...
    @staticmethod
    def transformFromNative(obj): ...



class StyleProxy:
    def __init__(self, target) -> None: ...
    def __getattr__(self, attr): ...
    def __setattr__(self, attr, value) -> None: ...
    def __copy__(self): ...
    def __add__(self, other): ...
    def copy(self, **kw): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



import sys
from typing import Callable
from typing_extensions import Literal

STDOUT: Literal[-11]
STDERR: Literal[-12]

if sys.platform == "win32":
    from ctypes import LibraryLoader, Structure, WinDLL, wintypes

    windll: LibraryLoader[WinDLL]
    COORD = wintypes._COORD

    class CONSOLE_SCREEN_BUFFER_INFO(Structure):
        dwSize: COORD
        dwCursorPosition: COORD
        wAttributes: wintypes.WORD
        srWindow: wintypes.SMALL_RECT
        dwMaximumWindowSize: COORD
    def winapi_test() -> bool: ...
    def GetConsoleScreenBufferInfo(stream_id: int = ...) -> CONSOLE_SCREEN_BUFFER_INFO: ...
    def SetConsoleTextAttribute(stream_id: int, attrs: wintypes.WORD) -> wintypes.BOOL: ...
    def SetConsoleCursorPosition(stream_id: int, position: COORD, adjust: bool = ...) -> wintypes.BOOL: ...
    def FillConsoleOutputCharacter(stream_id: int, char: str, length: int, start: COORD) -> int: ...
    def FillConsoleOutputAttribute(stream_id: int, attr: int, length: int, start: COORD) -> wintypes.BOOL: ...
    def SetConsoleTitle(title: str) -> wintypes.BOOL: ...

else:
    windll: None
    SetConsoleTextAttribute: Callable[..., None]
    winapi_test: Callable[..., None]



from typing import Any

signals_available: bool

class Namespace:
    def signal(self, name, doc: Any | None = ...): ...

class _FakeSignal:
    name: Any
    __doc__: Any
    def __init__(self, name, doc: Any | None = ...) -> None: ...
    send: Any
    connect: Any
    disconnect: Any
    has_receivers_for: Any
    receivers_for: Any
    temporarily_connected_to: Any
    connected_to: Any

scope_changed: Any



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class UpDownBars(Serialisable):
    tagname: str
    gapWidth: Any
    upBars: Any
    downBars: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self, gapWidth: int = ..., upBars: Any | None = ..., downBars: Any | None = ..., extLst: Any | None = ...
    ) -> None: ...



from typing import Any, Callable, TypeVar, overload
from typing_extensions import Literal

_F = TypeVar("_F", bound=Callable[..., Any])
_Actions = Literal["default", "error", "ignore", "always", "module", "once"]

class ClassicAdapter:
    reason: str
    version: str
    action: _Actions | None
    category: type[Warning]
    def __init__(
        self, reason: str = ..., version: str = ..., action: _Actions | None = ..., category: type[Warning] = ...
    ) -> None: ...
    def get_deprecated_msg(self, wrapped: Callable[..., Any], instance: object) -> str: ...
    def __call__(self, wrapped: _F) -> Callable[[_F], _F]: ...

@overload
def deprecated(__wrapped: _F) -> _F: ...
@overload
def deprecated(
    reason: str = ..., *, version: str = ..., action: _Actions | None = ..., category: type[Warning] | None = ...
) -> Callable[[_F], _F]: ...



from pathlib import Path

from .fpdf import FPDF as FPDF, TitleStyle as TitleStyle
from .html import HTML2FPDF as HTML2FPDF, HTMLMixin as HTMLMixin
from .template import Template as Template

__license__: str
__version__: str
FPDF_VERSION: str
FPDF_FONT_DIR: Path



import sys
from _tracemalloc import *
from typing import Any, Sequence, Union, overload
from typing_extensions import SupportsIndex

def get_object_traceback(obj: object) -> Traceback | None: ...
def take_snapshot() -> Snapshot: ...

class BaseFilter:
    inclusive: bool
    def __init__(self, inclusive: bool) -> None: ...

class DomainFilter(BaseFilter):
    @property
    def domain(self) -> int: ...
    def __init__(self, inclusive: bool, domain: int) -> None: ...

class Filter(BaseFilter):
    domain: int | None
    lineno: int | None
    @property
    def filename_pattern(self) -> str: ...
    all_frames: bool
    def __init__(
        self, inclusive: bool, filename_pattern: str, lineno: int | None = ..., all_frames: bool = ..., domain: int | None = ...
    ) -> None: ...

class Statistic:
    count: int
    size: int
    traceback: Traceback
    def __init__(self, traceback: Traceback, size: int, count: int) -> None: ...
    def __eq__(self, other: object) -> bool: ...

class StatisticDiff:
    count: int
    count_diff: int
    size: int
    size_diff: int
    traceback: Traceback
    def __init__(self, traceback: Traceback, size: int, size_diff: int, count: int, count_diff: int) -> None: ...
    def __eq__(self, other: object) -> bool: ...

_FrameTupleT = tuple[str, int]

class Frame:
    @property
    def filename(self) -> str: ...
    @property
    def lineno(self) -> int: ...
    def __init__(self, frame: _FrameTupleT) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __lt__(self, other: Frame) -> bool: ...
    if sys.version_info >= (3, 11):
        def __gt__(self, other: Frame) -> bool: ...
        def __ge__(self, other: Frame) -> bool: ...
        def __le__(self, other: Frame) -> bool: ...
    else:
        def __gt__(self, other: Frame, NotImplemented: Any = ...) -> bool: ...
        def __ge__(self, other: Frame, NotImplemented: Any = ...) -> bool: ...
        def __le__(self, other: Frame, NotImplemented: Any = ...) -> bool: ...

if sys.version_info >= (3, 9):
    _TraceTupleT = Union[tuple[int, int, Sequence[_FrameTupleT], int | None], tuple[int, int, Sequence[_FrameTupleT]]]
else:
    _TraceTupleT = tuple[int, int, Sequence[_FrameTupleT]]

class Trace:
    @property
    def domain(self) -> int: ...
    @property
    def size(self) -> int: ...
    @property
    def traceback(self) -> Traceback: ...
    def __init__(self, trace: _TraceTupleT) -> None: ...
    def __eq__(self, other: object) -> bool: ...

class Traceback(Sequence[Frame]):
    if sys.version_info >= (3, 9):
        @property
        def total_nframe(self) -> int | None: ...
        def __init__(self, frames: Sequence[_FrameTupleT], total_nframe: int | None = ...) -> None: ...
    else:
        def __init__(self, frames: Sequence[_FrameTupleT]) -> None: ...
    if sys.version_info >= (3, 7):
        def format(self, limit: int | None = ..., most_recent_first: bool = ...) -> list[str]: ...
    else:
        def format(self, limit: int | None = ...) -> list[str]: ...

    @overload
    def __getitem__(self, index: SupportsIndex) -> Frame: ...
    @overload
    def __getitem__(self, index: slice) -> Sequence[Frame]: ...
    def __contains__(self, frame: Frame) -> bool: ...  # type: ignore[override]
    def __len__(self) -> int: ...
    def __eq__(self, other: object) -> bool: ...
    def __lt__(self, other: Traceback) -> bool: ...
    if sys.version_info >= (3, 11):
        def __gt__(self, other: Traceback) -> bool: ...
        def __ge__(self, other: Traceback) -> bool: ...
        def __le__(self, other: Traceback) -> bool: ...
    else:
        def __gt__(self, other: Traceback, NotImplemented: Any = ...) -> bool: ...
        def __ge__(self, other: Traceback, NotImplemented: Any = ...) -> bool: ...
        def __le__(self, other: Traceback, NotImplemented: Any = ...) -> bool: ...

class Snapshot:
    def __init__(self, traces: Sequence[_TraceTupleT], traceback_limit: int) -> None: ...
    def compare_to(self, old_snapshot: Snapshot, key_type: str, cumulative: bool = ...) -> list[StatisticDiff]: ...
    def dump(self, filename: str) -> None: ...
    def filter_traces(self, filters: Sequence[DomainFilter | Filter]) -> Snapshot: ...
    @staticmethod
    def load(filename: str) -> Snapshot: ...
    def statistics(self, key_type: str, cumulative: bool = ...) -> list[Statistic]: ...
    traceback_limit: int
    traces: Sequence[Trace]



from typing import Any

from stripe.api_resources.abstract import (
    CreateableAPIResource as CreateableAPIResource,
    ListableAPIResource as ListableAPIResource,
    UpdateableAPIResource as UpdateableAPIResource,
    custom_method as custom_method,
)

class Payout(CreateableAPIResource, ListableAPIResource, UpdateableAPIResource):
    OBJECT_NAME: str
    def cancel(self, idempotency_key: Any | None = ..., **params): ...
    def reverse(self, idempotency_key: Any | None = ..., **params): ...



import sys
from _typeshed import Self
from types import TracebackType

class Timeout(TimeoutError):
    def __init__(self, lock_file: str) -> None: ...

class _Acquire_ReturnProxy:
    def __init__(self, lock: str) -> None: ...
    def __enter__(self) -> str: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, traceback: TracebackType | None
    ) -> None: ...

class BaseFileLock:
    def __init__(self, lock_file: str, timeout: float | int | str = ...) -> None: ...
    @property
    def lock_file(self) -> str: ...
    @property
    def timeout(self) -> float: ...
    @timeout.setter
    def timeout(self, value: int | str | float) -> None: ...  # type: ignore
    @property
    def is_locked(self) -> bool: ...
    def acquire(self, timeout: float | None = ..., poll_intervall: float = ...) -> _Acquire_ReturnProxy: ...
    def release(self, force: bool = ...) -> None: ...
    def __enter__(self: Self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, traceback: TracebackType | None
    ) -> None: ...
    def __del__(self) -> None: ...

class WindowsFileLock(BaseFileLock):
    def _acquire(self) -> None: ...
    def _release(self) -> None: ...

class UnixFileLock(BaseFileLock):
    def _acquire(self) -> None: ...
    def _release(self) -> None: ...

class SoftFileLock(BaseFileLock):
    def _acquire(self) -> None: ...
    def _release(self) -> None: ...

if sys.platform == "win32":
    FileLock = WindowsFileLock
elif sys.platform == "linux" or sys.platform == "darwin":
    FileLock = UnixFileLock
else:
    FileLock = SoftFileLock



from paramiko.message import Message
from paramiko.ssh_gss import _SSH_GSSAuth
from paramiko.transport import Transport

MSG_KEXGSS_INIT: int
MSG_KEXGSS_CONTINUE: int
MSG_KEXGSS_COMPLETE: int
MSG_KEXGSS_HOSTKEY: int
MSG_KEXGSS_ERROR: int
MSG_KEXGSS_GROUPREQ: int
MSG_KEXGSS_GROUP: int

c_MSG_KEXGSS_INIT: bytes
c_MSG_KEXGSS_CONTINUE: bytes
c_MSG_KEXGSS_COMPLETE: bytes
c_MSG_KEXGSS_HOSTKEY: bytes
c_MSG_KEXGSS_ERROR: bytes
c_MSG_KEXGSS_GROUPREQ: bytes
c_MSG_KEXGSS_GROUP: bytes

class KexGSSGroup1:
    P: int
    G: int
    b7fffffffffffffff: bytes
    b0000000000000000: bytes
    NAME: str
    transport: Transport
    kexgss: _SSH_GSSAuth
    gss_host: str | None
    x: int
    e: int
    f: int
    def __init__(self, transport: Transport) -> None: ...
    def start_kex(self) -> None: ...
    def parse_next(self, ptype: int, m: Message) -> None: ...

class KexGSSGroup14(KexGSSGroup1):
    P: int
    G: int
    NAME: str

class KexGSSGex:
    NAME: str
    min_bits: int
    max_bits: int
    preferred_bits: int
    transport: Transport
    kexgss: _SSH_GSSAuth
    gss_host: str | None
    p: int | None
    q: int | None
    g: int | None
    x: int | None
    e: int | None
    f: int | None
    old_style: bool
    def __init__(self, transport: Transport) -> None: ...
    def start_kex(self) -> None: ...
    def parse_next(self, ptype: int, m: Message) -> None: ...

class NullHostKey:
    key: str
    def __init__(self) -> None: ...
    def get_name(self) -> str: ...



from typing import Any

from .input_device import InputDevice as InputDevice
from .interaction import POINTER as POINTER, POINTER_KINDS as POINTER_KINDS

class PointerInput(InputDevice):
    DEFAULT_MOVE_DURATION: int
    type: Any
    kind: Any
    name: Any
    def __init__(self, kind, name) -> None: ...
    def create_pointer_move(self, duration=..., x: Any | None = ..., y: Any | None = ..., origin: Any | None = ...) -> None: ...
    def create_pointer_down(self, button) -> None: ...
    def create_pointer_up(self, button) -> None: ...
    def create_pointer_cancel(self) -> None: ...
    def create_pause(self, pause_duration) -> None: ...  # type: ignore[override]
    def encode(self): ...



from lib2to3.pgen2.grammar import _DFAS, Grammar
from lib2to3.pytree import _NL, _Convert, _RawNode
from typing import Any, Sequence, Text

_Context = Sequence[Any]

class ParseError(Exception):
    msg: Text
    type: int
    value: Text | None
    context: _Context
    def __init__(self, msg: Text, type: int, value: Text | None, context: _Context) -> None: ...

class Parser:
    grammar: Grammar
    convert: _Convert
    stack: list[tuple[_DFAS, int, _RawNode]]
    rootnode: _NL | None
    used_names: set[Text]
    def __init__(self, grammar: Grammar, convert: _Convert | None = ...) -> None: ...
    def setup(self, start: int | None = ...) -> None: ...
    def addtoken(self, type: int, value: Text | None, context: _Context) -> bool: ...
    def classify(self, type: int, value: Text | None, context: _Context) -> int: ...
    def shift(self, type: int, value: Text | None, newstate: int, context: _Context) -> None: ...
    def push(self, type: int, newdfa: _DFAS, newstate: int, context: _Context) -> None: ...
    def pop(self) -> None: ...



from typing import Any

from braintree.credit_card_verification import CreditCardVerification as CreditCardVerification
from braintree.errors import Errors as Errors

class ErrorResult:
    params: Any
    errors: Any
    message: Any
    credit_card_verification: Any
    transaction: Any
    subscription: Any
    merchant_account: Any
    def __init__(self, gateway, attributes) -> None: ...
    @property
    def is_success(self): ...



import sys
import types
from _typeshed import (
    OpenBinaryMode,
    OpenBinaryModeReading,
    OpenBinaryModeUpdating,
    OpenBinaryModeWriting,
    OpenTextMode,
    StrOrBytesPath,
    StrPath,
)
from abc import ABCMeta, abstractmethod
from importlib.machinery import ModuleSpec
from io import BufferedRandom, BufferedReader, BufferedWriter, FileIO, TextIOWrapper
from typing import IO, Any, BinaryIO, Iterator, Mapping, NoReturn, Protocol, Sequence, overload, runtime_checkable
from typing_extensions import Literal

_Path = bytes | str

class Finder(metaclass=ABCMeta): ...

class ResourceLoader(Loader):
    @abstractmethod
    def get_data(self, path: _Path) -> bytes: ...

class InspectLoader(Loader):
    def is_package(self, fullname: str) -> bool: ...
    def get_code(self, fullname: str) -> types.CodeType | None: ...
    def load_module(self, fullname: str) -> types.ModuleType: ...
    @abstractmethod
    def get_source(self, fullname: str) -> str | None: ...
    def exec_module(self, module: types.ModuleType) -> None: ...
    @staticmethod
    def source_to_code(data: bytes | str, path: str = ...) -> types.CodeType: ...

class ExecutionLoader(InspectLoader):
    @abstractmethod
    def get_filename(self, fullname: str) -> _Path: ...
    def get_code(self, fullname: str) -> types.CodeType | None: ...

class SourceLoader(ResourceLoader, ExecutionLoader, metaclass=ABCMeta):
    def path_mtime(self, path: _Path) -> float: ...
    def set_data(self, path: _Path, data: bytes) -> None: ...
    def get_source(self, fullname: str) -> str | None: ...
    def path_stats(self, path: _Path) -> Mapping[str, Any]: ...

# Please keep in sync with sys._MetaPathFinder
class MetaPathFinder(Finder):
    def find_module(self, fullname: str, path: Sequence[_Path] | None) -> Loader | None: ...
    def invalidate_caches(self) -> None: ...
    # Not defined on the actual class, but expected to exist.
    def find_spec(
        self, fullname: str, path: Sequence[_Path] | None, target: types.ModuleType | None = ...
    ) -> ModuleSpec | None: ...

class PathEntryFinder(Finder):
    def find_module(self, fullname: str) -> Loader | None: ...
    def find_loader(self, fullname: str) -> tuple[Loader | None, Sequence[_Path]]: ...
    def invalidate_caches(self) -> None: ...
    # Not defined on the actual class, but expected to exist.
    def find_spec(self, fullname: str, target: types.ModuleType | None = ...) -> ModuleSpec | None: ...

class Loader(metaclass=ABCMeta):
    def load_module(self, fullname: str) -> types.ModuleType: ...
    def module_repr(self, module: types.ModuleType) -> str: ...
    def create_module(self, spec: ModuleSpec) -> types.ModuleType | None: ...
    # Not defined on the actual class for backwards-compatibility reasons,
    # but expected in new code.
    def exec_module(self, module: types.ModuleType) -> None: ...

class _LoaderProtocol(Protocol):
    def load_module(self, fullname: str) -> types.ModuleType: ...

class FileLoader(ResourceLoader, ExecutionLoader, metaclass=ABCMeta):
    name: str
    path: _Path
    def __init__(self, fullname: str, path: _Path) -> None: ...
    def get_data(self, path: _Path) -> bytes: ...
    def get_filename(self, name: str | None = ...) -> _Path: ...
    def load_module(self, name: str | None = ...) -> types.ModuleType: ...

if sys.version_info >= (3, 7):
    class ResourceReader(metaclass=ABCMeta):
        @abstractmethod
        def open_resource(self, resource: StrOrBytesPath) -> IO[bytes]: ...
        @abstractmethod
        def resource_path(self, resource: StrOrBytesPath) -> str: ...
        if sys.version_info >= (3, 10):
            @abstractmethod
            def is_resource(self, path: str) -> bool: ...
        else:
            @abstractmethod
            def is_resource(self, name: str) -> bool: ...

        @abstractmethod
        def contents(self) -> Iterator[str]: ...

if sys.version_info >= (3, 9):
    @runtime_checkable
    class Traversable(Protocol):
        @abstractmethod
        def is_dir(self) -> bool: ...
        @abstractmethod
        def is_file(self) -> bool: ...
        @abstractmethod
        def iterdir(self) -> Iterator[Traversable]: ...
        @abstractmethod
        def joinpath(self, child: StrPath) -> Traversable: ...
        # The .open method comes from pathlib.pyi and should be kept in sync.
        @overload
        @abstractmethod
        def open(
            self,
            mode: OpenTextMode = ...,
            buffering: int = ...,
            encoding: str | None = ...,
            errors: str | None = ...,
            newline: str | None = ...,
        ) -> TextIOWrapper: ...
        # Unbuffered binary mode: returns a FileIO
        @overload
        @abstractmethod
        def open(
            self, mode: OpenBinaryMode, buffering: Literal[0], encoding: None = ..., errors: None = ..., newline: None = ...
        ) -> FileIO: ...
        # Buffering is on: return BufferedRandom, BufferedReader, or BufferedWriter
        @overload
        @abstractmethod
        def open(
            self,
            mode: OpenBinaryModeUpdating,
            buffering: Literal[-1, 1] = ...,
            encoding: None = ...,
            errors: None = ...,
            newline: None = ...,
        ) -> BufferedRandom: ...
        @overload
        @abstractmethod
        def open(
            self,
            mode: OpenBinaryModeWriting,
            buffering: Literal[-1, 1] = ...,
            encoding: None = ...,
            errors: None = ...,
            newline: None = ...,
        ) -> BufferedWriter: ...
        @overload
        @abstractmethod
        def open(
            self,
            mode: OpenBinaryModeReading,
            buffering: Literal[-1, 1] = ...,
            encoding: None = ...,
            errors: None = ...,
            newline: None = ...,
        ) -> BufferedReader: ...
        # Buffering cannot be determined: fall back to BinaryIO
        @overload
        @abstractmethod
        def open(
            self, mode: OpenBinaryMode, buffering: int = ..., encoding: None = ..., errors: None = ..., newline: None = ...
        ) -> BinaryIO: ...
        # Fallback if mode is not specified
        @overload
        @abstractmethod
        def open(
            self, mode: str, buffering: int = ..., encoding: str | None = ..., errors: str | None = ..., newline: str | None = ...
        ) -> IO[Any]: ...
        @property
        def name(self) -> str: ...
        @abstractmethod
        def __truediv__(self, child: StrPath) -> Traversable: ...
        @abstractmethod
        def read_bytes(self) -> bytes: ...
        @abstractmethod
        def read_text(self, encoding: str | None = ...) -> str: ...

    class TraversableResources(ResourceReader):
        @abstractmethod
        def files(self) -> Traversable: ...
        def open_resource(self, resource: StrPath) -> BufferedReader: ...  # type: ignore[override]
        def resource_path(self, resource: Any) -> NoReturn: ...
        def is_resource(self, path: StrPath) -> bool: ...
        def contents(self) -> Iterator[str]: ...
